#!/usr/bin/env python
"""
Calculates the dielectric constant of a fluid.
"""

import argparse

# system modules
import argparse
import math
import numpy as np

# custom modules
import f_core as fc
import output_tracker as ot

parser = argparse.ArgumentParser()
parser.add_argument('topology', type=str, help='The topology to work with.')
parser.add_argument('coordinates', type=str, help='The coordinates to work with.')
parser.add_argument('temperature', type=float, help='The simulation temperature.')

def magnetic_moments(universe, timestep, molecules):
	ms = np.zeros((len(molecules), 3))
	for i in range(len(molecules)):
		ms[i] = molecules[i].dipole_moment(universe, timestep)

	res = -np.sum((np.sum(ms, axis=1)/len(molecules))**2)
	res += np.sum(ms*ms)/len(molecules)

	return res

class molecule(object):
	def __init__(self, universe, ids):
		self._ids = ids
		self._charges = universe.atoms[ids].charges()*1.609*10e-19
		self._revmap = dict()
		for here, there in enumerate(self._ids):
			self._revmap[there] = here

		# fetch bonds
		self._bonds = []
		ag = universe.atoms[ids]
		self._ag = ag
		for key in ag.bondDict.keys():
			for bond in ag.bondDict[key]:
				self._bonds.append((bond.atom1.number, bond.atom2.number))

	def dipole_moment(self, universe, timestep):
		coord = self._ag.get_positions(ts=timestep)
		mu = np.zeros(3)
		for atom1, atom2 in self._bonds:
			mu += coord[self._revmap[atom1]]-coord[self._revmap[atom2]] * (self._charges[self._revmap[atom1]]-self._charges[self._revmap[atom2]])
		return mu

def main():
	"""
	Wrapper function.
	"""
	output = ot.output_tracker()
	mda = fc._require_mda(output)
	args = parser.parse_args()

	# messages
	messages = fc.messages

	# read input
	output.print_info('Reading source files.').add_level()
	output.print_info('Opening file %s.' % args.topology)
	output.print_info('Opening file %s.' % args.coordinates)

	# create Universe
	fc.mda_set_periodic(True, output)
	u_c = fc.mda_load_universe(args.topology, args.coordinates, output)
	output.print_info('Coordinate file holds %d frames.' % len(u_c.trajectory)).del_level()

	# build groups
	resids = set([_.resid for _ in u_c.atoms])
	output.print_info('Found %d residues. Using them as groups.' % len(resids))
	groups = []
	for resid in resids:
		groups.append(molecule(u_c, [_.number for _ in u_c.atoms if _.resid == resid]))

	# calculate epsilon
	for timestep in u_c.trajectory:
		m = magnetic_moments(u_c, timestep, groups)
		print 1+4*3.14/(3*timestep.volume*1.38*10e-23*args.temperature)*m
		

if __name__ == '__main__':
	main()


#!/usr/bin/env python
import sys 

class errors(object): #isdoc
	NO_MODE_AND_INPUT 			= (1, 'Mode and input file missing.')
	NO_OUTPUT 					= (2, 'No output file given.')
	OUTPUT_PRESENT 				= (3, 'Output file already present.')
	NO_Z_MATRIX 				= (4, 'No Z-matrix found.')
	UNEXPECTED_EOF 				= (5, 'Unexpected end of file.')
	INCOMPLETE_COORD 			= (6, 'Incomplete coordinate set.')
	PARSE_ERROR 				= (7, 'Parse error(s).')
	NO_CONFORMATIONS			= (8, 'No conformations found.')
	UNKNOWN_MODE        		= (9, 'Unknown operation mode.')
	MULTIPLE_KEYPRESS			= (10, 'Please stick to one key press at a time.')
	NO_SAMPLES					= (11, 'No samples to work with.')
	BEAT_MISMATCH 				= (12, 'Discarding samples due to beat count mismatch.')
	SAMPLE_NONEMPTY				= (13, 'Sample already recorded.')
	ONE_SAMPLE					= (14, 'Only one sample left.')
	PREMATURE_KEYPRESS  		= (15, 'First keypress before initialization was complete.')
	SAMPLE_EMPTY				= (16, 'Sample contains no beats.')
	USE_CACHED					= (17, 'Using cached values.')
	VIOLATED_CONSTRAINT 		= (18, 'Constraint violated.')
	CACHE_OUTDATED				= (19, 'Cached data is outdated.')
	INTERNAL_ERROR				= (20, 'Internal error.')
	UNSUPPORTED_FORMAT			= (21, 'Unsupported format.')
	POSSIBLE_ISSUE 				= (22, 'Possible issue. Please check.')
	MISSING_FRAMES				= (23, 'Unknown parameter frames.')
	MISSING_ATOMS				= (24, 'Unknown parameter frames.')
	NOT_IMPLEMENTED				= (25, 'Feature not implemented.')
	EMPTY_ATOM_LABEL			= (26, 'Empty atom label.')
	NO_PARAMETER_FILES			= (27, 'No force-field parameter files found.')
	CGENFF_OMITTED				= (28, 'Data from CGenFF found but omitted.')
	FILE_IGNORED				= (29, 'Ignoring file.')
	CHARMM_OMITTED				= (30, 'CHARMM parameter file omitted.')
	UNKNOWN_REQUEST				= (31, 'Read unknown request.')
	NO_D_MATRIX 				= (32, 'No distance matrix found.')
	NO_RESIDUE	 				= (33, 'At least one residue has to be specified.')
	TOO_MANY_RESIDUES			= (34, 'Only one residue can be analyzed at a time.')
	NO_REFERENCE_DATA			= (35, 'No reference data found.')
	NO_SCF_DATA					= (36, 'No SCF energies found.')
	NOT_CONVERGED				= (37, 'Geometric optimization did not converge.')
	GROUP_NOT_FOUND				= (38, 'Unable to select group.')
	AMBIGUOUS_KEY				= (39, 'Found multiple publications with the same citation key.')
	UNKNOWN_KEY					= (40, 'Found no publication with this citation key.')
	NO_DOI						= (41, 'No DOI entry found.')
	NO_JOURNAL					= (42, 'No journal entry found.')
	NO_ISSUE					= (43, 'No issue entry found.')
	NO_PAGES					= (44, 'No pages entry found.')
	NO_VOLUME					= (45, 'No volume entry found.')
	NO_YEAR						= (46, 'No year entry found.')
	NO_AUTHORS					= (47, 'No authors entry found.')
	BRACKET_MISMATCH			= (48, 'Different number of opening and closing brackets.')
	SYNTAX_COLON				= (49, 'Syntax error: wrong number of colons.')
	TARGET_VECTOR				= (50, 'Target vector not recognized.')
	BONDSPEC					= (51, 'Odd number of entries for bond specification.')
	BOND_CONV					= (52, 'Unable to convert bond indices to integers.')
	COMMAND						= (53, 'Error parsing command input.')
	UNKNOWN_ATOM_NAME			= (54, 'Atom name not found.')
	NO_SUCH_BOND				= (55, 'The given atoms are not connected by a bond.')
	UNKNOWN_VECTOR				= (56, 'The given vector is not a valid displacement.')
	ROTATION_I					= (57, 'The given rotation is the identity operation.')
	WRONG_RESIDUE				= (58, 'This atom name is only valid for another residue.')
	AMBIGUOUS_ATOM_NAME			= (59, 'This atom name is not unique for this residue or no residue given.')
	FIXED_RESIDUE				= (60, 'Atoms to rotate belong to a fixed residue.')
	NO_SUCH_RESIDUE				= (61, 'There is no residue with this name.')
	PSF_MISMATCH				= (62, 'The given PSF does not match the specified PDB file.')
	DOUBLE_PSF_SECTION			= (63, 'At least one section in the PSF is doubled.')
	PSF_BOND_NUMBER				= (64, 'Inconsistent number of bonds in PSF.')
	PSF_ATOM_NUMBER				= (65, 'Inconsistent number of atoms in PSF.')
	PSF_ATOM_UNIQUE				= (66, 'Non-unique atom number in PSF.')
	PSF_CONTIGUOUS				= (67, 'Atom numbers not contiguous in PSF.')
	ANGLESPEC					= (68, 'Number of entries for angles in PSF not a multiple of 3.')
	PSF_ANGLE_NUMBER			= (69, 'Inconsistent number of angles in PSF.')
	ANGLE_CONV					= (70, 'Unable to convert bond angle indices to integers.')
	DIHEDRAL_CONV				= (71, 'Unable to convert dihedral indices to integers. ')
	DIHEDRALSPEC				= (72, 'Number of entries for dihedrals in PSF not a multiple of 4.')
	PSF_DIHEDRAL_NUMBER 		= (73, 'Inconsistent number of dihedrals in PSF.')
	IMPROPERSPEC				= (74, 'Number of entries for impropers in PSF not a multiple of 4.')
	IMPROPER_CONV				= (75, 'Unable to convert improper indices to integers.')
	PSF_IMPROPER_NUMBER			= (76, 'Inconsistent number of impropers in PSF.')
	PSF_NO_ATOMS				= (77, 'PSF contains no atoms.')
	EMPTY_INPUT					= (78, 'Input contains no (readable) data.')
	PSF_REQUIRED				= (79, 'You have to load a PSF for this operation.')
	NAMD_NO_KEY_VAL				= (80, 'A line in the namd config file does not hold a key-value pair.')
	NAMD_NO_PDB					= (81, 'Namd config file does not reference any PDB files.')
	NAMD_MULTIPLE_PDB			= (82, 'Namd config file references multiple PDB files.')
	NAMD_NO_PSF					= (83, 'Namd config file does not reference any PSF.')
	NAMD_MULTIPLE_PSF			= (84, 'Namd config file references multiple PSF.')
	NAMD_NOT_CHARMM				= (85, 'Namd config file does not select CHARMM parameter format.')
	NAMD_NO_PRM					= (86, 'Namd config file does not reference any parameter files.')
	NO_NAMD_RUNFILE				= (87, 'No namd config file known.')
	NAMD_FAILED					= (88, 'Executing namd failed.')
	NAMD_CONVERGED				= (89, 'Optimization run did not converge.')
	PARSE_ENERGY				= (90, 'Failed to parse energy.')
	PDB_TOTAL_CHARGE			= (91, 'Total charge is nonzero.')
	CHARGE_PARSE				= (92, 'Unable to recognize charge request.')
	NO_SUCH_ATOM_TYPE			= (93, 'Unable to find the requested atom type.')
	NAMD_TRAJ_PARSE				= (94, 'Unable to parse namd trajectory.')
	EXPECTED_ATOM				= (95, 'Expected atom instance.')
	NAMD_NO_DCD					= (96, 'Namd config file does not reference any trajectory.')
	NAMD_MULTIPLE_DCD			= (97, 'Namd config file references multiple trajectories.')
	NAMD_NO_CONS_K_FILE			= (98, 'Namd config file does not reference a constraint forces file.')
	NAMD_MULTIPLE_CONS_K_FILE 	= (99, 'Namd config file references multiple constraint forces files.')
	NAMD_NO_CONS_FILE 			= (100, 'Namd config file does not reference a constraint file.')
	NAMD_MULTIPLE_CONS_FILE 	= (101, 'Namd config file references multiple constraint files.')
	UNKNOWN_INDEX				= (102, 'No atom with this index.')
	NO_CARTESIAN_VECTOR			= (103, 'The positional vector given is not a valid vector in cartesian coordinates.')
	NO_GAUSSIAN_OUTPUT			= (104, 'No Gaussian output files found.')
	UNKNOWN_VMD_IDENTIFIER		= (105, 'Atom identifier not found in output. Input file mismatch?')
	SELECTION_MISMATCH			= (106, 'Less atoms in output than requested.')
	VMD_FILENAME_DAMAGED		= (107, 'Unable to parse filename from VMD fftk.')
	ATOM_NUMBER_MISMATCH		= (108, 'Atom number in logfile does not match atom number in reference file.')
	GAU_NOT_ALIGNED				= (109, 'The analyzed bonds are not on the same axis. Constraint violation?')
	NAMD_NO_MINIMIZE			= (110, 'Namd config file does not contain any minimization request.')
	NAMD_MULTIPLE_MINIMIZE		= (111, 'Namd config file contains multiple minimization requests.')
	DISTANCE_CORRECTION			= (112, 'Initial align distance has to be corrected.')
	OMITTED_TAINTED_LINE		= (113, 'Omitting line containing tainted data.')
	OMITTED_DUPLICTED_POINT		= (114, 'Omitting line containing a variant of a previous datapoint.')
	LINE_NO_STRING				= (115, 'The given line is not convertible to a string.')
	NO_CHARMM_BOND_DEF			= (116, 'The given line does not contain a valid charmm bond definition.')
	KB_NO_FLOAT					= (117, 'Force constant non convertible to float.')
	B0_NO_FLOAT					= (118, 'Equilibrium distance non convertible to float.')
	DUPLICATE_ATOM_TYPE			= (119, 'Duplicate atom type.')
	EXPECTED_CHARMMFILE			= (120, 'Expected charmmfile instance.')
	AMBIGUOUS_ATOM_MERGE		= (121, 'Got ambiguous data for atom types during merge operation.')
	NO_CHARMM_ANGLE_DEF			= (122, 'The given line does not contain a valid charmm angle definition.')
	COMMENT_NO_STRING			= (123, 'Comment data not convertible to a string.')
	UB_DIST_NO_FLOAT			= (124, 'Urey-Bradley distance data not convertible to a float.')
	UB_KB_NO_FLOAT				= (125, 'Urey-Bradley force constant data not convertible to a float.')
	KB_NO_FLOAT					= (126, 'Force constant data not convertible to a float.')
	ANGLE_NO_FLOAT				= (127, 'Angle not convertible to a float.')
	DELTA_NO_FLOAT				= (128, 'Dihedral delta not convertible to a float.')
	MULTIPLICITY_NO_INT			= (129, 'Multiplicity not convertible to int.')
	NO_CHARMM_DIHEDRAL_DEF		= (130, 'The given line does not contain a valid charmm dihedral definition.')
	NO_CHARMM_IMPROPER_DEF		= (131, 'The given line does not contain a valid charmm improper definition.')
	NO_DATA_FOR_ELEMENT			= (132, 'No data found for this element name.')
	NO_DATA_FOR_ATOM_TYPE		= (133, 'No data found for this atom type.')
	CHARMMTOP_ATOM_DEF			= (134, 'Invalid ATOM definition in charmm topology file.')
	CHARMMTOP_BOND_DEF			= (135, 'Invalid BOND definition in charmm topology file.')
	CHARMMTOP_AMBIGUOUS_NAME	= (136, 'An atom name is used twice in a single residue.')
	CHARMMTOP_UNDEF_NAME		= (137, 'Charmm topology file references undefined atom name.')
	CHARMMTOP_PURELY_EXT		= (138, 'Purely external bonds not acceptable in charmm topology file.')
	RESNAME_NO_STRING			= (139, 'Residue name given is not convertible to a string.')
	EXPECTED_CHARMMRESFILE		= (140, 'Expected charmmresfile instance.')
	AMBIGUOUS_RESIDUE_MERGE		= (141, 'Got ambiguous data for residues during merge operation.')
	NO_TOPOLOGY_FILES			= (142, 'No topology data found.')
	NAMD_NO_COLVARS				= (143, 'Namd config file does not reference any collective variable file.')
	NAMD_MULTIPLE_COLVARS		= (144, 'Namd config file references multiple collective variable files.')
	DCD_NOT_FOUND				= (145, 'The specified DCD trajectory could not be read.')
	PSF_LINE_NO_CONV			= (146, 'Line count of PSF section is not an integer.')

# sanity check for error codes
codes = [x[1][0] for x in vars(errors).items() if x[0].upper() == x[0]]
if len(codes) != len(set(codes)):
	raise TypeError('Duplicate error code.')

class null_output(object): #isdoc
	def __getattribute__(self, name): #isdoc
		def print_error(error):  #isdoc
			raise Exception(error[1])

		if name == 'print_error':
			return print_error
		return lambda x=None: self

class output_tracker(object): #isdoc #doc-contains: _stdout, _indent, _indent_steps, _quiet, _colors
	def __enter__(self): #isdoc
		self._stdout = sys.stdout
		sys.stdout = self

	def __exit__(self, exc_type, exc_value, traceback): #isdoc
		sys.stdout = self._stdout

	def __init__(self): #isdoc
		self._indent = 0
		self._indent_steps = 3
		self._stdout = sys.stdout
		self._quiet = False

		# bash color codes
		self._colors = dict([
			('red', '0;31'), 
			('green', '0;32'),
			('brown', '0;33'),
			('default', '0')
			])

	def write(self, text): #isdoc
		self.print_info(text.strip('\n'))

	def flush(self): #isdoc
		pass

	def _print_multiline(self, prefix, message, color, overwrite=False):
		if len(prefix) not in  [0, 2]:
			raise ValueError('Internal Error: Wrong prefix length.')
		if prefix != '':
			prefix += ' '
		if type(message) != str:
			message = str(message)

		parts = message.split('\n')
		if len(parts) != 1:
			overwrite = False
		target = self._stdout
		if not target.isatty():
			target = sys.stderr
		if overwrite:
			target.write('\r')
		target.write('{1}\033[{3}m{0}\033[0m{2}'.format(
			prefix.upper(), 
			' ' * self._indent, 
			parts[0], 
			color
			))
		if overwrite:
			target.flush()
		else:
			target.write('\n')
		
		for part in parts[1:]:
			target.write(' {0}{1}\n'.format(' ' * (self._indent + 2), part))

	def print_error(self, code): #isdoc
		if not type(code[0]) is int:
			raise ValueError('Error code is not numeric.')
		self._print_multiline(
			'EE', 
			code[1] + (' (error code %d)' % code[0]), 
			self._colors['red']
			)
		raise Exception('Exception for traceback.')

	def print_warn(self, code): #isdoc
		if not type(code[0]) is int:
			raise ValueError('Warning code is not numeric.')
		self._print_multiline(
			'WW', 
			code[1] + (' (warning code %d)' % code[0]), 
			self._colors['brown']
			)
		return self

	def set_quiet(self, value): #isdoc
		self._quiet = value
		return self

	def get_quiet(self): #isdoc
		return self._quiet

	def print_info(self, message, parseable=None, overwrite=False):
		if not parseable is None:
			message = message % parseable

		if self._quiet:
			if not parseable is None:
				return self.print_plain(message)
			else:
				return self
		else:
			self._print_multiline('II', message, self._colors['green'], overwrite=overwrite)
			return self

	def add_level(self): #isdoc
		if not self._quiet:
			self._indent += self._indent_steps
		return self

	def del_level(self, depth=1): #isdoc
		if not self._quiet:
			self._indent -= self._indent_steps*depth
		if self._indent < 0:
			raise ValueError('Leftmost indentation level reached.')
		return self

	def print_plain(self, message): #isdoc
		self._print_multiline('', message, self._colors['default'])
		return self



# system modules
import math
import random
import types

# custom modules
import moremath

class atom: #isdoc #doc-contains: _comment, _idx, _atom_type, _label, _residue, _residue_id, _segment_name, _constrained, _charge, _mass, _psf_number, _pos, _charmm_idx
    def __str__(self): #isdoc
        prefix = 'atom' if self.get_name() is None else self.get_name()
        if self._pos is None:
            positions = '[no positions]'
        else:
            positions = '[x = {0:10f} y = {1:10f} z = {2:10f}]'.format(self._pos[0], self._pos[1], self._pos[2])
        return prefix + positions

    def __repr__(self): #isdoc
        return self.__str__()

    def __init__(self, box=None, label=None, idx=None): #isdoc
        if box is None:
            self._pos = None
        else:
            self.set_positions_by_box(self, box)
        self.set_label(label)
        self.set_name(idx)
        self.set_atom_type(None)
        self.set_comment(None)
        self._psf_number = None
        self._mass = None
        self._charge = None
        self._constrained = False
        self._segment_name = None
        self._residue_id = None
        self._charmm_idx = None

    def __sub__(self, other): #isdoc
        if self._pos is None:
            raise ValueError('No position specified.')
        if other._pos is None:
            raise ValueError('No position specified.')
        return math.sqrt(sum([(self._pos[i]-other._pos[i])**2 for i in range(3)]))

    def __add__(self, other): #isdoc
        if self._pos is None:
            raise TypeError('No position specified.')
        if type(other) == list:
            if len(other) == 3:
                self._pos = [sum(x) for x in zip(self._pos, other)]
            else:
                raise ArithmeticError('Unable to add different vectors of different lengths (lhs=%d, rhs=%d)' % (len(self._pos), len(other)))
        else:
            raise NotImplementedError()
        return self

    def __radd__(self, other): #isdoc
        return self + other

    def __deepcopy__(self, memo): #isdoc
        c = atom()
        c._pos = self._pos
        c._residue = self._residue
        c._label = self._label
        c._atom_type = self._atom_type
        c._idx = self._idx
        c._comment = self._comment
        c._psf_number = self._psf_number
        c._mass = self._mass
        c._charge = self._charge
        c._constrained = self._constrained
        c._segment_name = self._segment_name
        c._residue_id = self._residue_id
        c._charmm_idx = self._charmm_idx

        return c

    def set_residue(self, residue): #isdoc
        self._residue = residue
        return self

    def get_residue(self): #isdoc
        return self._residue

    def set_positions_by_box(self, box): #isdoc
        self._pos = [random.random()*box for i in range(3)]
        return self

    def set_positions(self, x, y=None, z=None): #isdoc
        # TODO: type checks
        if y is None and z is None and len(x) == 3:
            self._pos = list(x)
        else:
            self._pos = [x, y, z]
        return self

    def get_positions(self): #isdoc
        return self._pos

    def set_label(self, label): #isdoc
        if not type(label) in (str, types.NoneType):
            raise TypeError('Expected string or None as label, got %s.' % label)
        self._label = label
        return self

    def get_label(self): #isdoc
        return self._label

    def set_atom_type(self, atom_type): #isdoc
        if not type(atom_type) in (str, types.NoneType):
            raise TypeError('Expected string or None as atom type, got %s.' % atom_type)
        self._atom_type = str(atom_type)
        return self

    def get_atom_type(self): #isdoc
        return self._atom_type

    def set_name(self, idx): #isdoc
        self._idx = idx
        return self

    def get_name(self): #isdoc
        return self._idx

    def get_vector_to(self, other): #isdoc
        try:
            return map(lambda x: x[1]-x[0], zip(self._pos, other._pos))
        except:
            raise ValueError('No positional data defined.')

    def lj(self, other): #isdoc
        r = (self-other)**2
        if r < 0.1:
            return 1e6
        r = 1/r
        r *= r*r
        return (r*r-2*r)

    def set_comment(self, c): #isdoc
        if not type(c) in (str, types.NoneType):
            raise TypeError('Expected string or None as comment, got %s.' % c)
        self._comment = c
        return self

    def get_comment(self):  #isdoc
        return self._comment

    def translate(self, v): #isdoc
        self._pos = map(lambda x: sum(x), zip(self._pos, v))
        return self

    def rotate(self, n, angle): #isdoc
        self._pos = moremath.rotate_vector(self._pos, n, angle)
        return self

    def get_psf_number(self): #isdoc
        return self._psf_number

    def set_psf_number(self, n): #isdoc
        self._psf_number = int(n)
        return self

    def set_mass(self, n): #isdoc
        self._mass = n
        return self

    def get_mass(self): #isdoc
        return self._mass

    def set_charge(self, n): #isdoc
        self._charge = n
        return self

    def get_charge(self): #isdoc
        return self._charge

    def set_constrained(self, val): #isdoc
        self._constrained = bool(val)
        return self

    def is_constrained(self): #isdoc
        return self._constrained

    def set_segment_name(self, s): #isdoc
        self._segment_name = s
        return self

    def get_segment_name(self): #isdoc
        return self._segment_name

    def set_residue_id(self, i): #isdoc
        self._residue_id = int(i)
        return self

    def get_residue_id(self): #isdoc
        return self._residue_id

    def compatible_to(self, other): #isdoc
        if not isinstance(other, atom):
            return False

        checks = [atom.get_name, atom.get_residue, atom.get_segment_name]
        for check in checks:
            if not check(self) == check(other):
                return False
        return True

    def set_charmm_idx(self, idx): #isdoc
        self._charmm_idx = idx
        return self

    def get_charmm_idx(self): #isdoc
        return self._charmm_idx
        
elements = {
    1.008000: 'H', 
    12.011000: 'C', 
    14.007000: 'N', 
    15.999400: 'O', 
    15.999000: 'O', 
    30.974000: 'P',
    32.060000: 'S',
    18.998403: 'F'
    }

order_of_elements = 'H,He,Li,Be,B,C,N,O,F,Ne,Na,Mg,Al,Si,P,S,Cl,Ar,K,Ca,Sc,Ti,V,Cr,Mn,Fe,Co,Ni,Cu,Zn,Ga,Ge,As,Se,Br,Kr,Rb,Sr,Y,Zr,Nb,Mo,Tc,Ru,Rh,Pd,Ag,Cd,In,Sn,Sb,Te,I,Xe,Cs,Ba,La,Ce,Pr,Nd,Pm,Sm,Eu,Gd,Tb,Dy,Ho,Er,Tm,Yb,Lu,Hf,Ta,W,Re,Os,Ir,Pt,Au,Hg,Tl,Pb,Bi,Po,At,Rn,Fr,Ra,Ac,Th,Pa,U,Np,Pu,Am,Cm,Bk,Cf,Es,Fm,Md,No,Lr,Rf,Db,Sg,Bh,Hs,Mt,Ds,Rg,Cn,Uut,Fl,Uup,Lv,Uus,Uuo'.split(',')

def element_from_ordinal(no):
    try:
        no = int(no)
    except:
        raise TypeError('Ordinal not convertible to int.')
    
    if no < 1:
        raise ValueError('No such element.')
    if no > len(order_of_elements):
        raise ValueError('Unknown element.')

    return order_of_elements[no - 1]

def from_charmm_mass_definition(line): #isdoc
    try:
        parts = str(line).split()
    except:
        raise TypeError('Parameter not convertible to a string.')
    if len(parts) < 4:
        raise ValueError('Unable to parse line.')
    if parts[0] != 'MASS':
        raise ValueError('Line is no MASS definition.')

    try:
        mass = float(parts[3])
        label = elements[mass]
        idx = int(parts[1])
    except:
        raise ValueError('Unable to detect element label for mass %s' % parts[3])

    a = atom()
    a.set_atom_type(parts[2])
    a.set_label(label)
    a.set_mass(mass)
    a.set_charmm_idx(idx)
    comment = ' '.join(parts[4:]).strip()
    if len(comment) > 0:
        if not comment.startswith('!'):
            raise ValueError('Unable to parse what is supposed to be a comment.')
        comment = comment[1:]
    else:
        comment = ''
    a.set_comment(comment)

    return a

def from_pdb_line(line): #isdoc
    #ATOM     11  F3  LIG M   1       2.758   2.773   0.275  0.00  0.00      MOL  F

    if not line.startswith('ATOM') and not line.startswith('HETATM'):
        raise ValueError('Non-Atom line given.')

    try:
        label = line[76:78].strip()
    #    1/len(label)
    except:
        raise ValueError('Missing atom label.')

    try:
        residue = line[17:20].strip()
        1/len(residue)
    except:
        raise ValueError('Missing residue name.')

    c = atom(label=label)
    c.set_residue(residue)
    try:
        x = float(line[30:38])
        y = float(line[38:46])
        z = float(line[46:54])
    except:
        raise ValueError('Unable to convert atom coordinates.')

    c.set_positions(x, y, z)
    c.set_segment_name(line[72:76].strip())
    try:
        c.set_name(line[12:16].strip())
    except:
        raise ValueError('Unable to extract atom name.')

    return c

def from_psf_line(line): #isdoc
    #1 MOL      1        LIG      C1       CC33A  -0.270000       12.0107           0
    parts = line.strip().split()
    c = atom()
    c.set_residue(parts[3])
    c.set_name(parts[4])
    c.set_mass(float(parts[7]))
    c.set_charge(float(parts[6]))
    c.set_constrained(float(parts[8]))
    c.set_psf_number(int(parts[0]))
    c.set_segment_name(parts[1])
    c.set_residue_id(int(parts[2]))
    c.set_atom_type(parts[5])

    return c


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# system modules
import codecs
import copy
import datetime
import logging
try:
	logging.captureWarnings(True)
except:
	pass # < python 2.7
import math
import re

# custom modules
import atom
import colorexcept
import output_tracker as ot

# third-party modules
try:
	import MDAnalysis as mda
	HAS_MDA = True
except ImportError:
	HAS_MDA = False
import numpy as np
import numpy.linalg as npl

class messages(object):
	"""
		Error and warning messages. Used to make error codes unique throughout all tools of the finley toolkit.
	"""
	IMPORT_FAILED 		= (1, 'Unable to import topology and coordinate files.')
	NO_SLICE 			= (2, 'Slice indices have to be either integers or have to be of zero-length.')
	SLICE_START_RANGE 	= (3, 'Slice start selects frames that are not part of the source trajectory.')
	SLICE_STOP_RANGE 	= (4, 'Slice stop selects frames that are not part of the source trajectory.')
	NO_FRAME 			= (5, 'No frame selected.')
	OUTPUT_EXISTS 		= (6, 'Output file already exists.')
	CHARMM_SELECTION 	= (7, 'Unable to parse CHARMM selection.')
	OUTPUT_FAILED		= (8, 'Unable to write to output file.')
	INVALID_FRAME_SEL 	= (9, 'Frame not within trajectory.')
	MANUAL_SEEK			= (10, 'Performing stepwise seek in file for frame selection.')
	SEEK_PAST_FILE		= (11, 'End of file occurred during seeking. File may be corrupted.')
	MDA_DEP				= (12, 'MDAnalysis is required for this routine. You may continue after its installation.')
	UNKNOWN_LENGTH		= (13, 'Unable to estimate the total length of a trajectory.')
	EMPTY_SELECTION		= (14, 'No atoms selected.')
	INT_CONVERSION 		= (15, 'Could not convert data to integer.')
	SELECT_FAILED 		= (16, 'Unable to select requested atoms.')
	NEGATIVE_ATOM_NUM 	= (17, 'No negative atom numbers supported.')
	RANGE_ATOM_NUM 		= (18, 'Atom number is out of range.')
	NO_DUPLICATE 		= (19, 'All atom numbers in the selection have to be unique.')
	ORIGIN_LENGTH 		= (20, 'Origin has to be tuple of length three.')
	INVALID_ORIGIN 		= (21, 'Cannot parse specified origin.')
	NO_CMD				= (22, 'You have to specify at least one command to get any output.')
	TERNARY_CMD			= (23, 'All commands require at most two colon-separated sections, got more.')
	NOT_CONSUMED		= (24, 'RPN stack not fully consumed. Perhaps you left out an operator.')
	UNKNOWN_CMD			= (25, 'Unknown command.')
	REVERSE_BOUNDS		= (26, 'Reverse bounds for range.')
	STACK_POP			= (27, 'Stack does not hold enough elements for this operator.')
	STACK_DIMENSION		= (28, 'Elements on stack have wrong dimension for this operator.')
	DIM_MISMATCH		= (29, 'Elements have mismatching dimensions.')
	DASH_RANGE_NONNEG	= (30, 'Negative indices not supported for dash range syntax.')
	DASH_RANGE_SYNTAX	= (31, 'Syntax error for dash range selection.')
	NO_NULL_VECTOR		= (32, 'Null vectors are not allowed for this operation.')
	BOUNDS_REVERSED 	= (33, 'Lower limit is higher than the upper one.')
	BOUNDS_EQUAL		= (34, 'Lower and upper limits are identical.')
	BIN_COUNT_LEQ_Z		= (35, 'Number of bins has to be at least one.')
	EMPTY_LINE			= (36, 'Empty or ill-formatted input line.')
	CONVERT_FAILED		= (37, 'Got non-numeric data.')
	OUT_OF_BOUNDS		= (38, 'Data out of bounds.')
	BOUNDS_DIMENSION	= (39, 'Bound specification has to have the same dimension as the input data.')
	BOUNDS_IGNORED		= (40, 'Bound specification ignored.')
	BINS_2D_SPEC		= (41, 'Bins for the x axis and the y axis for 2D data have to be separated by a semicolon.')
	BINS_SORTED			= (42, 'Bins were not sorted.')
	UNDEF_PLANE			= (43, 'At least three points in space required for plane definition.')
	NOT_EXIST			= (44, 'File does not exist or is not readable.')
	IDX_SYNTAX_ERROR	= (45, 'GROMACS-style index file has syntax errors.')
	IDX_EMPTY_GROUP		= (46, 'GROMACS-style index file has empty index group name.')
	IDX_GROUP_SPACE		= (47, 'GROMACS-style index file has index group name containing whitespace.')
	IDX_MINVAL			= (48, 'GROMACS-style index files do not allow for indices lower than 1.')
	IDX_GROUP_OVERRIDE	= (49, 'Multiple definition of index group.')
	NEGATIVE_LENGTH		= (50, 'Unable to accept negative length specification.')
	UNDEF_DIHEDRAL		= (51, 'Dihedral definition requires four atoms to be selected.')
	VEL_COORD_MISMATCH	= (52, 'Different number of frames for velocities and coordinates.')
	IDENTICAL_GROUPS	= (53, 'Unable to evaluate the forces between identical groups.')


def _parse_indexfile(lines, groups, output):
	"""
	Graceful parser for GROMACS-style index files. In GROMACS, atom numbering is one-based. This routine works only on one-based indices.

	Parameters
	----------
	lines : List of strings
		Input lines of the file
	groups : Dictionary
		Group data like the return format of this function.
	output : :class:`output_tracker` instance
		Error reporting.

	Returns
	-------
	Dictionary
		Keys are case-sensitive group names as strings. Values are lists of integers. 
	"""
	groupname = None
	groupnumbers = set()

	def _append_group(groupname, groups, output, groupnumbers):
		if groupname in groups:
			output.print_error(messages.IDX_GROUP_OVERRIDE)
		groups[groupname] = groupnumbers
		return None, set()

	for line in lines:
		line = line.strip()
		if len(line) == 0:
			continue

		if line.startswith('['):
			if groupname is not None:
				groupname, groupnumbers = _append_group(groupname, groups, output, groupnumbers)
			if not line[-1] == ']':
				output.print_error(messages.IDX_SYNTAX_ERROR)
			groupname = line[1:-1].strip()
			if len(groupname) == 0:
				output.print_error(messages.IDX_EMPTY_GROUP)
			if ' ' in groupname or '\t' in groupname:
				output.print_error(messages.IDX_GROUP_SPACE)
		else:
			if groupname is None:
				output.print_error(messages.IDX_SYNTAX_ERROR)
			parts = line.split()
			try:
				parts = map(int, parts)
			except:
				output.print_info('Parsing line: %s' % line)
				output.print_error(messages.INT_CONVERSION)
			for part in parts:
				if part < 1:
					output.print_info('Got %d' % part)
					output.print_error(messages.IDX_MINVAL)
			groupnumbers = groupnumbers.union(set(parts))

	if groupname is not None:
		_append_group(groupname, groups, output, groupnumbers)
	return groups

def parse_indexfiles(filenames, output):
	"""
	Graceful parser of GROMACS-style index files.

	Parameters
	----------
	filenames : List of strings
		Input filenames.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	Dictionary
		Keys are case-sensitive group names as strings. Values are lists of integers. Atom indices are one-based.
	"""
	groups = dict()
	if filenames is None:
		return groups

	for filename in filenames:
		try:
			fh = open(filename, 'r')
		except:
			output.print_info('Reading file %s' % filename)
			output.print_error(messages.NOT_EXIST)
		groups = _parse_indexfile(fh.readlines(), groups, output)

	return groups

def parse_frame_selection(frames, output):
	"""
	Takes a pythonic selection syntax as string and converts it to a :func:`slice` object.

	Parameters
	----------
	frames : String
	  Selection command for integer ranges.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	slice
		A :func:`slice` object.

	Examples
	--------
	>>> parse_frame_selection('1:2:10', output)
	slice(1, 2, 10)

	>>> parse_frame_selection('', output)
	slice(None, None, None)

	>>> parse_frame_selection('-1', output)
	slice(-1, None, 1)

	>>> parse_frame_selection('::10', output)
	slice(None, None, 10)
	"""
	f_sel = slice(None)
	if frames == '':
		return f_sel
	frames = frames.strip()
	if ':' in frames:
		parts = frames.split(':')
		try:
			parts = map(lambda x: None if x.strip() == '' else int(x), parts)
		except:
			output.print_info('Given frame slice was %s' % frames)
			output.print_error(messages.NO_SLICE)
		f_sel = slice(*parts)
	else:
		try:
			num = int(frames)
		except:
			output.print_info('Given frame slice was %s' % frames)
			output.print_error(messages.NO_SLICE)
		if num >= 0:
			f_sel = slice(num, num+1)
		elif num == -1:
			f_sel = slice(-1, None, 1)
		else:
			f_sel = slice(num, num+1, 1)

	return f_sel

def _require_mda(output):
	"""
	Makes sure that MDAnalysis is loaded and prints an error message otherwise.

	Raises
	------
	Exception
		If MDAnalysis is unavailable. 

	Returns
	-------
	module
		MDAnalysis module instance.
	"""
	if not HAS_MDA:
		output.print_error(messages.MDA_DEP)

	return mda

def mda_goto(universe, target_frame, output):
	"""
	Seeks to a specific frame in the trajectory.

	Parameters
	----------
	universe : :class:`mda:MDAnalysis.core.AtomGroup.Universe`
	  The universe which trajectory is to be analyzed.
	target_frame : integer
	  Zero-based index of the frame to select.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	integer:
	  Zero-based index of the frame that was selected when this function was called.
	"""
	_require_mda(output)

	# MDAnalysis uses one based frame numbers
	target_frame += 1
	initial_frame = universe.trajectory.frame

	if target_frame == initial_frame:
		return

	try:
		if target_frame >= len(universe.trajectory):
			output.print_error(messages.INVALID_FRAME_SEL)
	except:
		# implementing __len__ for MDAnalysis trajectories is optional
		pass

	try:
		universe.trajectory[target_frame]
	except:
		# seek not implemented in MDAnalysis
		output.print_warn(messages.MANUAL_SEEK)
		if target_frame < current_frame:
			universe.trajectory.rewind()

		try:
			while universe.trajectory.frame < target_frame:
				universe.trajectory.next()
		except IOError:
			output.print_error(messages.SEEK_PAST_FILE)

	return initial_frame - 1

def _parse_dash_range(command, output):
	"""
	Parses range specification based on dash-separated limits.

	Parameters
	----------
	command : String
		The selection to parse.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	Iterable :
		List of integers.

	Examples
	--------
	>>> _parse_dash_range('2-3')
	[2, 3]
	"""
	parts = command.split('-')
	if len(parts) != 2:
		output.print_info('Got %s.' % parts)
		output.print_error(messages.DASH_RANGE_SYNTAX)

	try:
		temp = map(int, parts)
	except:
		output.print_info('Got %s.' % command)
		output.print_error(messages.INT_CONVERSION)
	start, stop = temp
	if stop < start:
		output.print_info('Got %s.' % temp)
		output.print_error(messages.REVERSE_BOUNDS)
	if start < 0 or stop < 0:
		output.print_error(messages.DASH_RANGE_NONNEG)

	return range(start, stop+1)

def _parse_colon_range(command, output):
	"""
	Parses range specification in python syntax.

	Parameters
	----------
	command : String
		The selection to parse.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	Iterable :
		List of integers.	

	Examples
	--------
	>>> _parse_colon_range('2:5:2')
	[2, 4]
	"""
	parts = command.split(':')
	if len(parts) not in (2, 3):
		output.print_info('Got %s.' % command)
		output.print_error(messages.COLON_RANGE_SYNTAX)

	try:
		temp = map(int, parts)
	except:
		output.print_info('Got %s.' % command)
		output.print_error(messages.INT_CONVERSION)

	return range(*temp)

def _parse_ids(command, output):
	"""
	Parses id selections including mixed range specifications, exclusions and precedence.

	Ranges can be given either by dash separated limits (includes the upper bound) or by python-like colon separated limits (does not include the upper limit). Stride syntax supported for the latter. Open ranges are not allowed.

	Parameters
	----------
	command : String
		The selection to parse
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	Iterable :
		Sorted list of integers.

	Examples
	--------
	>>> _parse_ids('3-6')
	[3, 4, 5, 6]
	>>> _parse_ids('3-6,^4-5')
	[3, 6]
	>>> _parse_ids('2-3,4:7:2')
	[2, 3, 4, 6]
	"""
	# build list from range selections
	command = command.strip()
	cmds = command.replace(' ', ',').split(',')

	# filter repeated separators
	cmds = [_ for _ in cmds if len(_) != 0]

	# loop over commands
	ids = set()
	for cmd in cmds:
		exclude = cmd.startswith('^')
		if exclude:
			cmd = cmd[1:]

		if '-' in cmd:
			cmd_ids = _parse_dash_range(cmd, output)
		elif ':' in cmd:
			cmd_ids = _parse_colon_range(cmd, output)
		else:
			try:
				cmd_ids = set([int(cmd)])
			except:
				output.print_info('Got %s.' % cmd)
				output.print_error(messages.INT_CONVERSION)

		if exclude:
			ids = ids.difference(cmd_ids)
		else:
			ids = ids.union(cmd_ids)

	return list(ids)

def mda_select(universe, selection, output, selection_frame=None):
	"""
	Selects an atom group by CHARMM selection or :ref:`atom selection <atomselections>` syntax.

	Parameters
	----------
	universe : :class:`mda:MDAnalysis.core.AtomGroup.Universe`
	  The universe which trajectory is to be analyzed.
	selection : String
	  The CHARMM-style selection request.
	output : :class:`output_tracker` instance
	  Error reporting.
	selection_frame : Integer, optional
	  If set, the selection command will be performed using the coordinates of this frame.

	Returns
	-------
	MDAnalysis.atomGroup
	  Atom group which is valid for all frames of the loaded trajectory of the given universe.

	Raises
	------
	Exception
		If MDAnalysis is unavailable.
	"""
	_require_mda(output)

	if selection is None:
		initial_selection = universe.atoms 
	elif selection.startswith('id '):
		ids = _parse_ids(selection[3:], output)
		try:
			ag = universe.atoms[ids]
		except Exception, e:
			output.print_info('MDAnalysis: %s' % str(e))
			output.print_error(messages.SELECT_FAILED)
	else:
		if selection_frame is not None:
			previous = mda_goto(universe, selection_frame, output)
		try:
			initial_selection = universe.selectAtoms(selection)
		except Exception, e:
			raise
			output.print_info('MDAnalysis: %s' % str(e))
			output.print_error(messages.CHARMM_SELECTION)
		if selection_frame is not None:
			mda_goto(universe, previous, output)
	return initial_selection

if HAS_MDA:
	class IdSelection(mda.core.Selection.Selection):
		"""
		Parser for id selection.
		"""
		def __init__(self, ids):
			"""
			Accepts the data to be parsed.

			Parameters
			----------
			ids : String
				Ids. See :ref:`atomselections`.
			"""
			self._data = ids

		def _apply(self, group):
			"""
			Parses the data given.

			Parameters
			----------
			group : :class:`MDAnalysis.core.AtomGroup`
				Atoms to select from.

			Returns
			-------
			Set 
				Selected Atoms.

			Raises
			------
			ValueError
				If the selection was unparseable.
			"""
			output = ot.null_output()			
			try:
				ids = _parse_ids(self._data, output)
			except:
				raise ValueError('Unable to parse ID selection %s' % ids)
			atoms = set()
			for e in ids:
				atoms.insert(groups.atoms[e])
			return atoms

	class GroupSelection(mda.core.Selection.Selection):
		"""
		Parser for named group selection.
		"""
		def __init__(self, groups):
			"""
			Accepts the data to be parsed.

			Parameters
			----------
			groups : String
				Comma-separated group names without any spaces.
			"""
			self._data = groups

		def _apply(self, group):
			"""
			Parses the data given.

			Parameters
			----------
			group : :class:`MDAnalysis.core.AtomGroup`
				Atoms to select from.

			Returns
			-------
			Set 
				Selected Atoms.

			Raises
			------
			TypeError
				If the atom group has not been initially selected from an :class:`UniverseWrapper` instance.
			ValueError
				If any group is unknown or no group was given.
			"""
			try:
				available_groups = group.universe.get_index_groups()
			except:
				raise TypeError('group selection requires the usage of UniverseWrapper instead of Universe.')

			groupnames = self._data.split(',')
			available_names = set(available_groups.keys())
			requested_names = set(groupnames)
			if len(requested_names-available_names) > 0:
				raise ValueError('Unknown group(s): %s' % ', '.join(list(requested_names-available_names)))
			if len(requested_names) == 0:
				raise ValueError('At least one group has to be specified.')

			indices = [set(available_groups[_]) for _ in requested_names]
			indices = reduce(lambda x, y: x.union(y), indices)
			return set([_ for _ in group.atoms if _.number in indices])

	class UniverseWrapper(mda.core.AtomGroup.Universe):
		"""
		Allows for additional information to be connected to an MDAnalysis :class:`~MDAnalysis.core.AtomGroup.Universe`.
		"""
		#: Named groups. Keys: Strings of group names, Values: List of one-based atom indices
		_index_groups = dict()

		def get_index_groups(self):
			"""
			Retrieves named atom groups.

			Returns
			-------
			Dictionary
				Keys are group names, Values are iterables of one-based atom indices.
			"""
			return self._index_groups

		def add_index_groups(self, groups):
			"""
			Adds multiple named atom groups to the topology definition.

			Parameters
			----------
			groups : Dictionary
				Keys are group names, Values are iterables of one-based atom indices.
			"""
			for group, indices in groups.iteritems():
				try:
					self._index_groups[str(group)] = [int(_) for _ in indices]
				except:
					# No output_tracker connection here for better transferability.
					raise ValueError('Invalid group dictionary entries.')

		def remove_index_groups(self, groups, require_existing=False):
			"""
			Removes multiple named atom groups from the topology definition.

			Parameters
			----------
			groups : Iterable
				Group names to remove.
			require_existing : Boolean
				Whether to raise an exception if any of the specified groups has not been defined so far.
			"""
			for group in groups:
				try:
					del self._index_groups[str(group)]
				except:
					if require_existing:
						raise ValueError('Group %s does not exist.' % group)

	class ExtendedSelectionParser(mda.core.Selection.SelectionParser):
		"""
		Extends the MDAnalysis selection parser.

		.. note:: 
			Has to be an old-style class for compatibility to MDAnalysis.
		"""

		#: Selection token for atom ids
		ID = 'id'
		#: Selection token for named groups
		GROUP = 'groups'

		def __init__(self):
			"""
			Adds the newly defined keywords to the keywords :class:`mda.core.Selection.SelectionParser` supports.
			"""
			self.classdict[self.ID] = IdSelection
			self.classdict[self.GROUP] = GroupSelection

		def _SelectionParser__parse_subexp(self):
			"""
			Intercepts parsing of subexpressions and deals with the newly defined selection commands.
			"""
			op = self._SelectionParser__peek_token()

			# idea copied from mda.core.Selection.SelectionParser.__parse_subexp
			if op in (self.ID, self.GROUP):
				op = self._SelectionParser__consume_token()
				data = self._SelectionParser__consume_token()
				if data in (self.LPAREN, self.RPAREN, self.AND, self.OR, self.NOT, self.SEGID, self.RESID, self.RESNAME, self.NAME, self.TYPE):
					self._SelectionParser__error("Identifier")
				return self.classdict[op](data)
			else:
				return mda.core.Selection.SelectionParser._SelectionParser__parse_subexp(self)

def mda_patch_parser(enable=True):
	"""
	Patches the loaded MDAnalysis selection parser to incorporate `group` and `id` keywords.

	Parameters
	----------
	enable : Boolean
		Whether to activate the extended parser. If `False`, the original state will be recovered.
	"""
	if enable:
		mda.core.Selection.Parser = ExtendedSelectionParser()
	else:
		mda.core.Selection.Parser = mda.core.Selection.SelectionParser()

def mda_select_id(universe, selection, output):
	"""
	Extends the MDAnalysis CHARMM selection parser by a keyword for selection by id. 

	Atom ids are one-based (as usual for MDAnalysis).

	Parameters
	----------
	universe : :class:`mda:MDAnalysis.core.AtomGroup.Universe`
	  The universe which trajectory is to be analyzed.
	selection : String
	  The CHARMM-style selection request.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	MDAnalysis.atomGroup
	  Atom group which is valid for all frames of the loaded trajectory of the given universe.
	"""
	if selection.startswith('id '):
		ids = _parse_ids(selection[3:], output)
		try:
			ag = universe.atoms[ids]
		except Exception, e:
			output.print_info('MDAnalysis: %s' % str(e))
			output.print_error(fc.messages.SELECT_FAILED)
	else:
		if len(selection.strip()) == 0:
			selection = 'all'
		ag = mda_select(universe, selection, output)

	return ag

def mda_set_periodic(value, output):
	"""
	Sets the flag whether to respect periodic boundary conditions in MDAnalysis calculations.

	Parameters
	----------
	value : Boolean
	  Flag value

	Raises
	------
	Exception
		If MDAnalysis is unavailable.
	"""
	_require_mda(output)

	mda.core.flags['use_periodic_selections'] = value
	mda.core.flags['use_KDTree_routines'] = (not value)

def mda_load_universe(topology, coordinates, output):
	"""
	Creates a MDAnalysis.Universe from a trajectory with error checks.

	Parameters
	----------
	topology : String
	  Topology file name. Supported file formats: see MDAnalysis. May be compressed with gzip or bzip2.
	coordinates : String
	  Coordinate file name. Supported file formats: see MDAnalysis. May be compressed with gzip or bzip2.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	universe
	  The created :class:`UniverseWrapper` which is essentially identical to :class:`mda:MDAnalysis.core.AtomGroup.Universe`.

	Raises
	------
	Exception
		If MDAnalysis is unavailable.
	Exception
		If the import failed.
	"""
	_require_mda(output)

	try:
		u = UniverseWrapper(topology, coordinates)
	except Exception, e:
		output.print_info('MDAnalysis: %s' % str(e))
		output.print_error(messages.IMPORT_FAILED)

	return u

def mda_get_framecount(universe, selection, output, complain_empty=False):
	"""
	Calculates the number of frames selected by a slice for MDAnalysis trajectories.

	Parameters
	----------
	universe : :class:`mda:MDAnalysis.core.AtomGroup.Universe`
	  The universe which trajectory is to be analyzed.
	selection : slice
	  A frame selection.
	output : :class:`output_tracker` instance
	  Error reporting.
	complain_empty: Boolean, optional
	  Whether to report a fatal error on empty selection.

	Returns
	-------
	Integer
	  The number of frames.

	Raises
	------
	Exception
		If MDAnalysis is unavailable.
	Exception
		If the slice is of length zero and `complain_empty` is True.
	"""
	_require_mda(output)

	# implementing __len__ for MDAnalysis trajectories is optional
	try:
		length = len(universe.trajectory)
	except:
		output.print_warn(messages.UNKNOWN_LENGTH)
		return None

	start = 0 if selection.start is None else selection.start if selection.start >= 0 else length + selection.start
	stop = length if selection.stop is None else selection.stop if selection.stop >= 0 else length + selection.stop
	step = 1 if selection.step is None else abs(selection.step)
	numframes = max((stop-start)/step, 0)
	if numframes == 0 and complain_empty:
		output.print_error(messages.NO_FRAME)
	if abs(start) > length:
		output.print_error(messages.SLICE_START_RANGE)
	if abs(stop) > length:
		output.print_error(messages.SLICE_STOP_RANGE)

	return numframes

def dot_product(a, b, output):
	""" Calculates the dot product of two vectors of arbitrary dimensionality.

	Parameters
	----------
	a, b : iterable, numeric
		Vectors. 
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	The dot product.

	Raises
	------
	Exception:
		When the vectors have different dimensions.
	"""
	a = np.array(a)
	b = np.array(b)

	if len(a) != len(b):
		output.print_error(messages.DIM_MISMATCH)

	return np.array(a).dot(np.array(b))

def angle(a, b, output):
	"""
	Calculates the angle between two vectors of arbitrary dimensionality.

	Parameters
	----------
	a, b : iterable, numeric
		Vectors.
	output : :class:`output_tracker` instance
	  Error reporting.

	Returns
	-------
	The angle in radians.

	Raises
	------
	Exception:
		When the vectors have different dimensions.
	Exception:
		When at least one of the vectors is the null vector.
	"""
	a = np.array(a)
	b = np.array(b)
	if npl.norm(a) == 0 or npl.norm(b) == 0:
		output.print_error(messages.NO_NULL_VECTOR)

	return math.acos(dot_product(a, b, output) / (npl.norm(a)*npl.norm(b)))

class PSFFile(object):#doc-contains: _angles, _atoms, _bonds, _dihedrals, _impropers, _lines, _output, _parsed
	"""
	Holds information about a single protein structure format file. Implements a fluent interface.
	"""
	def __init__(self, stream, output=None):
		"""
		Prepares data structures.

		.. todo:: 
			Check for attribute documentation.

		Parameters
		----------
		stream : List of strings or file object.
			Input data.
		output : :class:`output_tracker` or `None`
			Error reporting. If parameter value is `None`, error reporting is disabled.
		"""
		#: Reference to the currently used :class:`~output_tracker.output_tracker` instance.
		self._output = None
		if output is None:
			self._output = ot.null_output()
		else:
			self._output = output

		#: List of strings. ASCII lines in the input file.
		self._lines = []
		if type(stream) == list:
			self._lines = stream
		else:
			self._lines = stream.readlines()
		if len(self._lines) == 0 or not type(self._lines[0]) is str:
			self._output.print_error(ot.errors.EMPTY_INPUT)

		#: Internal flag whether the data in :attr:`_lines` has been parsed already to allow for lazy parsing.
		self._parsed = False
		#: List of sets containing the indices of the :class:`~toto.atom.atom` instances in :attr:`_atoms` that form a bond.
		self._bonds = [] 
		#: List of :class:`~toto.atom.atom` instances. Ordering is relevant for :attr:`_angles`, :attr:`_bonds`, :attr:`_dihedrals`, and :attr:`_impropers`.
		self._atoms = [] 
		#: List of sets containing the indices of the :class:`~toto.atom.atom` instances in :attr:`_atoms` that form an angle.
		self._angles = [] 
		#: List of sets containing the indices of the :class:`~toto.atom.atom` instances in :attr:`_atoms` that form a dihedral.
		self._dihedrals = []
		#: List of sets containing the indices of the :class:`~toto.atom.atom` instances in :attr:`_atoms` that form an improper dihedral.
		self._impropers = []

	def __deepcopy__(self, memo):
		"""
        Creates a completely independent instance of :class:`psffile` that essentially is identical to the current one.

        Parameters
        ----------
        memo : Dictionary
        	Dictionary of objects already copied.

        Returns
        -------
        :class:`psffile`
        	The copy.
		"""
		c = psffile(self._lines, self._output)
		c._parsed = self._parsed
		c._bonds = self._bonds[:]
		c._atoms = [copy.deepcopy(_) for _ in self._atoms]
		c._angles = self._angles[:]
		c._dihedrals = self._dihedrals[:]
		c._impropers = self._impropers[:]
		return c

	def parse(self):
		"""
		Cached parsing of the given source lines.

		Returns
		-------
		self
			Allows for fluent interface.
		"""
		if self._parsed:
			return self

		self._parse_atoms()._parse_bonds()._parse_angles()._parse_dihedrals()._parse_impropers()
		self._parsed = True
		return self

	def get_total_charge(self):
		"""
        Calculates the total charge of all atoms.

        Returns
        -------
        Float
        	Charge in electron charges.
		"""
		return sum([_.get_charge() for _ in self._atoms])

	def get_total_mass(self):
		"""
        Calculates the total mass of all atoms.

        Returns
        -------
        Float
        	Mass in amu.
		"""
		return sum([_.get_mass() for _ in self._atoms])

	def _scan_section(self, header):
		"""
        Reads all lines for a given section of a PSF file.
		
		.. note:: 
        	This routine performs no checks whatsoever on this header entry.

        Parameters
        ----------
        header : String
        	The header section to search for.

        Returns
        -------
        Tuple
        	Tuple of the number of entries that should be in the file and a list of strings holding all input lines for this section. Please note that empty entries can occurr in case no lines for this section could be found.
		"""
		started = False
		section = []
		header = '!' + header
		expected_num = None
		for line in self._lines:
			if header in line:
				started = True
				if expected_num is not None:
					self._output.print_error(ot.errors.DOUBLE_PSF_SECTION)
				try:
					expected_num = int(line.strip().split(header)[0])
				except:
					self._output.print_error(ot.errors.PSF_LINE_NO_CONV)
				continue
			if started and line.strip() == '':
				started = False

			if started:
				section.append(line)

		return (expected_num, section)

	def _parse_bonds(self):
		"""
        Extracts all data from :attr:`_lines` that is related to bonds.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		enum, bondspec = self._scan_section('NBOND')
		bondspec = ' '.join([_.strip() for _ in bondspec])

		bondspec = bondspec.split()
		if len(bondspec) % 2 != 0:
			self._output.print_error(ot.errors.BONDSPEC)
		try:
			bondspec = map(int, bondspec)
			bondspec = map(lambda x: x-1, bondspec)
		except:
			self._output.print_error(ot.errors.BOND_CONV)
		f = bondspec[0::2]
		t = bondspec[1::2]

		self._bonds = zip(f, t)
		if len(self._bonds) != enum:
			self._output.print_error(ot.errors.PSF_BOND_NUMBER)

		return self

	def _parse_atoms(self):
		"""
        Extracts all data from :attr:`_lines` that is related to atoms.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		enum, atomspec = self._scan_section('NATOM')
		# TODO: check for enum=None which means no NATOM section
		if len(atomspec) != enum:
			self._output.print_info('Expected %d atoms from header, got %d lines.' % (enum, len(atomspec)))
			self._output.print_error(ot.errors.PSF_ATOM_NUMBER)

		self._atoms = [atom.from_psf_line(_) for _ in atomspec]
		atom_numbers = [_.get_psf_number() for _ in self._atoms]
		if len(self._atoms) == 0:
			self._output.print_error(ot.errors.PSF_NO_ATOMS)

		if len(set(atom_numbers)) != len(atom_numbers):
			self._output.print_error(ot.errors.PSF_ATOM_UNIQUE)

		if not min(atom_numbers) == 1 and max(atom_numbers) == len(atom_numbers) + 1:
			self._output.print_error(ot.errors.PSF_CONTIGUOUS)
		return self

	def _parse_angles(self):
		"""
		Extracts all data from :attr:`_lines` that is related to angles.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		enum, anglespec = self._scan_section('NTHETA')
		anglespec = ' '.join([_.strip() for _ in anglespec])

		angles = anglespec.split()
		if len(angles) % 3 != 0:
			self._output.print_error(ot.errors.ANGLESPEC)
		try:
			angles = map(int, angles)
			angles = map(lambda x: x-1, angles)
		except:
			self._output.print_error(ot.errors.ANGLE_CONV)
		f = angles[0::3]
		v = angles[1::3]
		t = angles[2::3]

		self._angles = zip(f, v, t)
		if len(self._angles) != enum:
			self._output.print_error(ot.errors.PSF_ANGLE_NUMBER)
		return self

	def _parse_dihedrals(self):
		"""
        Extracts all data from :attr:`_lines` that is related to dihedrals.

        Returns
        -------
        self
			Allows for fluent interface.
		"""
		enum, dhspec = self._scan_section('NPHI')
		dhspec = ' '.join([_.strip() for _ in dhspec])

		dihedrals = dhspec.split()
		if len(dihedrals) % 4 != 0:
			self._output.print_error(ot.errors.DIHEDRALSPEC)
		try:
			dihedrals = map(int, dihedrals)
			dihedrals = map(lambda x: x-1, dihedrals)
		except:
			self._output.print_error(ot.errors.DIHEDRAL_CONV)
		f 	= dihedrals[0::4]
		v1 	= dihedrals[1::4]
		v2 	= dihedrals[2::4]
		t 	= dihedrals[3::4]

		self._dihedrals = zip(f, v1, v2, t)
		if len(self._dihedrals) != enum:
			self._output.print_error(ot.errors.PSF_DIHEDRAL_NUMBER)
		return self

	def _parse_impropers(self):
		"""
        Extracts all data from :attr:`_lines` that is related to improper dihedrals.

        Returns
        -------
        self
			Allows for fluent interface.
		"""
		enum, impspec = self._scan_section('NIMPHI')
		impspec = ' '.join([_.strip() for _ in impspec])

		impropers = impspec.split()
		if len(impropers) % 4 != 0:
			self._output.print_error(ot.errors.IMPROPERSPEC)
		try:
			impropers = map(int, impropers)
			impropers = map(lambda x: x-1, impropers)
		except:
			self._output.print_error(ot.errors.IMPROPER_CONV)
		f 	= impropers[0::4]
		v1 	= impropers[1::4]
		v2 	= impropers[2::4]
		t 	= impropers[3::4]

		self._impropers = zip(f, v1, v2, t)
		if len(self._impropers) != enum:
			self._output.print_error(ot.errors.PSF_IMPROPER_NUMBER)
		return self

	def delete_atom(self, idx):
		"""
		Removes all data related to a specific atom from the internal representation.

		Parameters
		----------
		idx : Integer
        	The list index of the :class:`~toto.atom.atom` instances in :attr:`_atoms`.

        Returns
        -------
        self
			Allows for fluent interface.
		"""
		for atm in self._atoms[idx+1:]:
			atm.set_psf_number(atm.get_psf_number()-1)
		del self._atoms[idx]

		for group in [self._bonds, self._angles, self._dihedrals, self._impropers]:
			for gnr, entry in enumerate(group):
				if idx in entry:
					group[gnr] = None
					continue
				ecopy = [_-1 if _ > idx else _ for _ in entry]
				entry = tuple(ecopy)
				group[gnr] = entry
		self._bonds = [_ for _ in self._bonds if not _ is None]
		self._angles = [_ for _ in self._angles if not _ is None]
		self._dihedrals = [_ for _ in self._dihedrals if not _ is None]
		self._impropers = [_ for _ in self._impropers if not _ is None]
		return self

	def _delete_residues_except(self, residue):
		"""
        Removes all entries that are related to all but a single residue.

        Parameters
        ----------
        residue : String
        	Residue name of the only residue that is to be kept.

        Returns
        -------
        self :
        	Allows for fluent interface.
		"""
		tbd = [idx for idx, _ in enumerate(self._atoms) if _.get_residue() != residue]
		tbd.reverse()
		for a in tbd:
			self.delete_atom(a)
		return self

	def extract_residue(self, residue):
		"""
        Builds a new :class:`psffile` instance only containing a single residue.

        Parameters
        ----------
        residue : String
        	Residue name.

        Returns
        -------
        :class:`psffile`
        	A completely independent instance.
		"""
		check = [_ for _ in self._atoms if _.get_residue() == residue]
		if len(check) == 0:
			self._output.print_error(ot.errors.NO_SUCH_RESIDUE)

		n = copy.deepcopy(self)
		n._delete_residues_except(residue)
		return n

	def get_psf_string(self):
		"""
        Builds a string representation of the whole file.

        Returns
        -------
        String
        	Whole file a single string. Lines are separated by newline characters.
		"""
		ret = 'PSF\n\n       1 !NTITLE\n REMARKS generated on ' + str(datetime.datetime.now()) + '\n\n'
		ret += '%8d !NATOM\n' % len(self._atoms)
		for a in self._atoms:
			ret += '{0:8} {1:4} {2:<4} {3:4} {4:4} {5:>4} {6:> 10.6f} {7:>15.6f}  {8:8}\n'.format(
				a.get_psf_number(),
				a.get_segment_name(),
				a.get_residue_id(),
				a.get_residue(),
				a.get_name(),
				a.get_atom_type(),
				a.get_charge(),
				a.get_mass(),
				int(a.is_constrained()))

		for label, data in [('NBOND', self._bonds), ('NTHETA', self._angles), ('NPHI', self._dihedrals), ('NIMPHI', self._impropers)]:
			ret += '\n%8d !%s:\n' % (len(data), label)
			dlist = [''.join(['{0:8}'.format(_+1) for _ in elements]) for elements in data]
			#dlist = [list(_) for _ in data]
			#dlist = ['{0:8}'.format(_+1) for _ in sum(dlist, [])]
			tline = ''
			for element in dlist:
				if len(tline) > 60:
					ret += tline + '\n'
					tline = ''
				tline += element
			ret += tline + '\n'

		ret += '\n       0 !NDON:\n\n       0 !NACC:\n\n       0 !NNB:\n\n'
		return ret

	def is_compatible(self, atomlist):
		"""
        Checks whether there is a unique one-to-one mapping of all atoms in the internal data structures. 

        For the definition of compatible, see :meth:`~toto.atom.atom.compatible_to`.

        Parameters
        ----------
        atomlist : List of :class:`~toto.atom.atom` instances
        	The atoms to compare to. The ordering of the atoms is of relevance to the result as the PSF indices are projected onto this list.

        Returns
        -------
        Boolean
        	True only if no inconsistencies occurred during the check.
		"""
		if len(self._atoms) != len(atomlist):
			return False

		for a1, a2 in zip(self._atoms, atomlist):
			if not a1.compatible_to(a2):
				return False

		return True

	def is_bonded(self, f, t):
		"""
        Checks whether a bond between two :class:`~toto.atom.atom` instances is defined in this instance.

        Ordering does not matter.

        Parameters
        ----------
        f : :class:`~toto.atom.atom`
        	First atom of the bond.
        t : :class:`~toto.atom.atom`
        	Last atom of the bond.
        	
        Returns
        -------
        Boolean
		"""
		return (f, t) in self._bonds or (t, f) in self._bonds

	def is_dihedral(self, f, v1, v2, t):
		"""
		Checks whether a torsional angle between four :class:`~toto.atom.atom` instances is defined in this instance.

        Ordering does matter. Either `f`-`v1`-`v2`-`t` or `t`-`v2`-`v1`-`f` have to exist in case this function evaluates to True.

        Parameters
        ----------
        f : :class:`~toto.atom.atom`
        	First atom of the angle.
        v1 : :class:`~toto.atom.atom`
        	Middle atom of the angle.
        v2 : :class:`~toto.atom.atom`
        	Middle atom of the angle.
        t : :class:`~toto.atom.atom`
        	Last atom of the angle.
        	
        Returns
        -------
        Boolean
		"""
		return (f, v1, v2, t) in self._dihedrals or (t, v2, v1, f) in self._dihedrals

	def is_improper(self, f, v1, v2, t):
		"""
		Checks whether an improper torsional angle between four :class:`~toto.atom.atom` instances is defined in this instance.

        Ordering does matter. Either `f`-`v1`-`v2`-`t` or `t`-`v2`-`v1`-`f` have to exist in case this function evaluates to True.

        Parameters
        ----------
        f : :class:`~toto.atom.atom`
        	First atom of the angle.
        v1 : :class:`~toto.atom.atom`
        	Middle atom of the angle.
        v2 : :class:`~toto.atom.atom`
        	Middle atom of the angle.
        t : :class:`~toto.atom.atom`
        	Last atom of the angle.
        	
        Returns
        -------
        Boolean
		"""
		return (f, v1, v2, t) in self._impropers or (t, v2, v1, f) in self._impropers

	def is_angle(self, f, v, t):
		"""
        Checks whether an angle between three :class:`~toto.atom.atom` instances is defined in this instance.

        Ordering does matter. Either `f`-`v`-`t` or `t`-`v`-`f` have to exist in case this function evaluates to True.

        Parameters
        ----------
        f : :class:`~toto.atom.atom`
        	First atom of the angle.
        v : :class:`~toto.atom.atom`
        	Middle atom of the angle.
        t : :class:`~toto.atom.atom`
        	Last atom of the angle.
        	
        Returns
        -------
        Boolean
		"""
		return (f, v, t) in self._angles or (t, v, f) in self._angles

	def set_charge(self, atom_type, charge):
		"""
        Changes the internal value of the charge of a certain atom type.

        Parameters
        ----------
        atom_type : String
        	Atom type selection of the relevant atoms.
        charge : Float
        	New charge in electron charges.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		for a in self._atoms:
			if a.get_atom_type() == atom_type:
				a.set_charge(charge)

		return self

	def get_atom(self, idx):
		"""
		Access method for a single atom.

		Parameters
		----------
		idx : Integer
			Zero-based index of the atom to retrieve.

		Returns
		-------
		:class:`~toto.atom.atom`
			The atom instance.

			.. note::
				Please keep in mind that this atom instance is still related to the :class:`psffile` instance you got it from. In case you want to modify the atom properties but leave the PSF file unchanged, you have to call :meth:`~todo.atom.atom.__deepcopy__`.
		"""
		return self._atoms[idx]

	def get_atom_count(self):
		"""
		Returns the number of atoms that are described in this PSF file.
		"""
		return len(self._atoms)

	def iteratoms(self):
		"""
		Generator for iterating over the atoms of this PSF file.
		"""
		for _ in self._atoms:
			yield _

class PDBFile(object): #isdoc #doc-contains: _psf, _lines, _parsed, _atoms, _valid, _residues, _output
	"""
	Holds information about a single protein database file.
	"""

	def __init__(self, stream, output = None):
		"""
		Prepares internal data structures.

		Parameters
		----------
		stream : List of strings, file object
			Input file as list of the input lines or a file object.
		output : :class:`output_tracker` or `None`
			Error reporting. If parameter value is `None`, error reporting is disabled.
		"""
		#: Reference to either :class:`~output_tracker.output_tracker` or :class:`~output_tracker.null_output`. In the latter case only error messsages are printed. Is to be set by :meth:`__init__`.
		self._output = None
		if output is None:
			self._output = ot.null_output()
		else:
			self._output = output

		#: List of strings. Contains all data from the source file stream. Should be considered read-only.
		self._lines = None
		if type(stream) == list:
			self._lines = stream
		else:
			self._lines = stream.readlines()
		if len(self._lines) == 0 or not type(self._lines[0]) is str:
			self._output.print_error(ot.errors.EMPTY_INPUT)

		self._strip_bom()

		#: Dictionary used for consistency checks during strict parsing. The residue names as strings are used as keys. Each entry contains another dictionary holding the :class:`~toto.atom.atom` instances as values using their (supposedly) unique atom name as key.
		self._residues = dict()
		#: Holds the result of internal linter. A value of `None` denotes that no syntax check has been perfomed, yet. Otherwise this attribute holds a boolean value. Some of the access methods use a separate parsing method that is less strict.
		self._valid = None
		#: List of :class:`~toto.atom.atom` instances. Is kept in sync with the atom list of :attr:`_psf`. May be empty. 
		self._atoms = []
		#: A reference to :class:~toto.pdbfile.psffile`. If no PSF file is loaded, this reference is `None`.
		self._psf = None
		#: Boolean. Used for caching. Is `True` when the source data in :attr:`_lines` has been parsed.
		self._parsed = False

	def purge_metadata(except_first=False):
		"""
		Removes all lines from the internal representation that do not contain atom definitions. 

		Practically, this means removing all lines that do not contain a ATOM or HETATM definition.

		Parameters
		----------
		except_first : Boolean
			Whether to keep the first contiguous set of non-atom lines that precedes any atom definition.

		Returns
		-------
		self :
			Allows for fluent interface.
		"""
		offset = 0
		atom_lines = [idx for idx, _ in enumerate(self._lines) if _.startswith('ATOM') or _.startswith('HETATM')]
		if except_first:
			offset = min(atom_lines)
		self._lines = self._lines[0:offset] + [self._lines[_] for _ in atom_lines]

		return self

	def find_atoms(self, residue_name=None, atom_name=None):
		"""
        Creates a list of the atoms in this PDB file only containing the atoms with a certain residue name or atom name.

        Parameters
        ----------
        residue_name : String
        	The residue name to filter for. `None` matches all atoms.
        atom_name : String
        	The atom name to filter for. `None` matches all atoms.

        Returns
        -------
        List of :class:`~toto.atom.atom` 
        	Instances that fulfill all specified criteria.
		"""
		res = self._atoms[:]

		if not residue_name is None:
			res = [_ for _ in res if _.get_residue() == residue_name]

		if not atom_name is None:
			res = [_ for _ in res if _.get_name() == atom_name]

		return list(res)

	def get_bond(self, bond):
		"""
        Extracts the atoms that form a specific bond.
		
		.. todo:: 
			Check whether the two atoms are in fact bonded. 

        Parameters
        ----------
        bond : String
        	Format: `RESIDUE_NAME.ATOM_NAME-RESIDUE_NAME.ATOM_NAME`. The residue name may be omitted in case the atom name is unique to the whole file.
      
      	Returns
      	-------
      	List of :class:`~toto.atom.atom`.
      		Length two. Instances that match the request.
		"""
		parts = bond.split('-')
		if len(parts) != 2:
			self._output.print_error(ot.errors.PARSE_ERROR)

		atoms = []
		for part in parts:
			e = part.split('.')
			if len(e) == 1:
				residue_name = None
				atom_name = e[0]
			if len(e) == 2:
				residue_name, atom_name = e
			if len(e) > 2:
				self._output.print_error(ot.errors.PARSE_ERROR)
			r = self.find_atoms(residue_name, atom_name)
			if len(r) == 0:
				self._output.print_info('Looking for %s' % part)
				self._output.print_error(ot.errors.UNKNOWN_ATOM_NAME)
			if len(r) > 1:
				self._output.print_info('Looking for %s' % part)
				self._output.print_error(ot.errors.AMBIGUOUS_ATOM_NAME)
			atoms.append(r[0])
		return atoms

	def get_pdb_string(self, renumber=False, clean=False):
		"""
        Creates a new PDB string representation that could be written to a file.

        .. todo:: 
        	Currently, this ignores changes in atom count and bond topology.

        Parameters
        ----------
        renumber : Boolean
        	In case any of the atoms are removed, the atom number should be changes such that all atoms are numbered starting from one and in a contiguous fashion. As this alters the references in the specific file and potentially makes the new PDB file incompatible with other external resources, this behavior is deactivated by default.

        Returns
        -------
        String
        	Lines separated by newline characters.
		"""
		res = ''
		idx = 0
		for line in self._lines:
			if line.startswith('ATOM') or line.startswith('HETATM'):
				label = line[12:16].strip()
				a = self._atoms[idx].get_positions()
				if renumber:
					atom_number = str(idx+1)
				else:
					atom_number = line[6:11]
				idx += 1
				line = '{0}{1:5}{2}{3:8.3f}{4:8.3f}{5:8.3f}{6}'.format(line[0:6], atom_number, line[11:30], a[0], a[1], a[2], line[54:])
				res += line
			else:
				if clean is False:
					res += line
		if clean is True:
			res += 'END'
		return res

	def get_psf_string(self):
		"""
        Interface method for creation of a string representation of the PSF file that is related to the given PDB file. Essentially calls :meth:`PSFFile.get_psf_string`.

        Returns
        -------
        String
        	Lines separated by newline characters.
		"""
		if self._psf is None:
			self._output.print_error(ot.errors.PSF_REQUIRED)

		return self._psf.get_psf_string()

	def get_total_charge(self):
		"""
        Calculates the total charge of all atoms.

        Returns
        -------
        Float
        	Charge in electron charges.
		"""
		if self._psf is None:
			self._output.print_error(ot.errors.PSF_REQUIRED)
			
		return self._psf.get_total_charge()

	def get_total_mass(self):
		"""
        Calculate the total mass of all atoms.

        Returns
        -------
        Float
        	Mass in amu.
		"""
		return self._psf.get_total_mass()

	def set_charge(self, atom_type, charge):
		"""
        Changes the internal value of the charge of a certain atom type.

        Parameters
        ----------
        atom_type : String
        	The atom type to filter for.
        charge : Float
        	New value in electron charges.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		if self._psf is None:
			self._output.print_error(ot.errors.PSF_REQUIRED)
			
		self._psf.set_charge(atom_type, charge)
		return self

	def get_atom_count(self, atom_type=None):
		"""
        Counts the atoms with a specific atom type. 

        If `atom_type` is `None`, the total atom number is returned. For atom type filtering, a PSF file has to be loaded. The total number of all atoms can be determined without having a matching PSF file.

        Parameters
        ----------
        atom_type : String or None
        	The atom type to filter for.

        Returns
        -------
        Integer
        	Atom type occurrence.
		"""
		if atom_type is None:
			return len(self._atoms)

		if self._psf is None:
			self._output.print_error(ot.errors.PSF_REQUIRED)

		return len([_ for _ in self._psf._atoms if _.get_atom_type() == atom_type])

	def get_all_atom_types(self):
		"""
        Extracts a list of all atom types without any duplicates.

        Returns
        -------
        Set
        	Unique enumeration of all atom types.
		"""
		if self._psf is None:
			self._output.print_error(ot.errors.PSF_REQUIRED)

		return set([_.get_atom_type() for _ in self._psf._atoms])

	def __deepcopy__(self, memo):
		"""
        Helper method for copying a whole PDB file including any depending PSF.

        Parameters
        ----------
        memo : dictionary
        	Contains all objects that have been copied already.

        Returns
        -------
        :class:`PDBFile`
      		A new and completely independent instance.
		"""
		c = pdbfile(self._lines, self._output)
		c._residues = copy.deepcopy(self._residues)
		c._valid = self._valid
		c._residues = copy.deepcopy(self._residues)
		c._atoms = [copy.deepcopy(_) for _ in self._atoms]
		c._parsed = self._parsed
		c._psf = copy.deepcopy(self._psf)
		return c

	def _delete_residues_except(self, residue):
		"""
		Removes all entries that are related to all but a single residue.

		Parameters
		----------
		residue : String
			Residue name of the only residue that is to be kept.

		Returns
		-------
		self
			Allows for fluent interface.
		"""
		# remove invalid lines
		nl = []
		for line in self._lines:
			if not line.startswith('ATOM ') and not line.startswith('HETATM'):
				nl.append(line)
			else:
				if line[17:20].strip() == residue:
					nl.append(line)
		self._lines = nl

		# remove invalid atoms
		for idx, atm in enumerate(self._atoms):
			if atm.get_residue() != residue:
				self._atoms[idx] = None
		self._atoms = [_ for _ in self._atoms if _ is not None]

		self._psf = self._psf.extract_residue(residue)

	def extract_residue(self, residue):
		"""
        Builds a new :class:`PDBFile` instance only containing a single residue.
      
      	Parameters
      	----------
      	residue : String
      		Residue name of the only residue that is to be kept.

      	Returns
      	-------
      	:class:`PDBFile`
      		An independent instance.
		"""
		if not residue in self.get_residues():
			self._output.print_error(ot.errors.NO_SUCH_RESIDUE)

		n = copy.deepcopy(self)
		n._delete_residues_except(residue)
		return n

	def parse(self):
		"""
        Reads the input lines in :attr:`_lines`.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		if self._parsed:
			return self
		lines = self._get_atom_lines()
		try:
			for line in lines:
				self._atoms.append(atom.from_pdb_line(line))
		except Exception, e:
			self._output.print_info('Parsing line %s' % line.strip())
			self._output.print_info(str(e))
			self._output.print_error(ot.errors.PARSE_ERROR)
		self._parsed = True
		return self

	def read_psf(self, stream):
		"""
		Creates a new :class:`psffile` from the data given in `stream`. The new internal :class:`psffile` instance is connected to the same :class:`~output_tracker.output_tracker` as `self`. Parsing applies relaxed standards.

		Parameters
		----------
		stream : List of strings, file object.
			Input lines in either format.

		Returns
		-------
		self
			Allows for fluent interface.
		"""
		self.parse()
		self._psf = psffile(stream, self._output)
		self._psf.parse()
		#if not self._psf.is_compatible(self._atoms):
		#	self._output.print_error(ot.errors.PSF_MISMATCH)

	def get_atom(self, index):
		"""
        Access method.

        Parameters
        ----------
        index : Integer
			Zero based index of the selected atom in :attr:`_atoms`.

		Returns
		-------
		:class:`~toto.atom.atom`
			Requested instance.
		"""
		try:
			return self._atoms[index]
		except:
			self._output.print_info('Retrieving atom with index %d' % index)
			self._output.print_error(ot.errors.UNKNOWN_INDEX)

	def get_index(self, name, residue=None):
		"""
        Extracts the index of a certain atom by atom name and residue.

        Parameters
        ----------
        name : String
        	Atom name
        residue : String
        	Residue name

        Returns
        -------
        Integer
        	Zero based index of the selected atom in :attr:`_atoms`.
		"""
		if residue is None:
			parts = name.split('.')
			if len(parts) == 2:
				residue, name = parts

		c = [(idx, a) for idx, a in enumerate(self._atoms) if a.get_name() == name]
		if len(c) == 0:
			self._output.print_error(ot.errors.UNKNOWN_ATOM_NAME)
		if residue is not None:
			c = [(idx, a) for idx, a in c if a.get_residue() == residue]
		if len(c) == 0:
			self._output.print_error(ot.errors.WRONG_RESIDUE)
		if len(c) > 1:
			self._output.print_error(ot.errors.AMBIGUOUS_ATOM_NAME)
		return c[0][0]

	def get_bond_vector(self, f, t, crucial=False):
		"""
        Calculates the direction of a bond. 

        Parameters
        ----------
        f : Integer
        	First atom index.
        t : Integer
        	Last atom index.
        crucial : Boolean
        	If `False`, calling this function with two nonbonded atoms will only raise a warning. If `True`, an error will occur in this case.

        Returns
        -------
        Tuple of floats
        	Bond vector in three dimensions.
		"""
		atom_f = self._atoms[f]
		atom_t = self._atoms[t]
		if not self._psf.is_bonded(f, t):
			if crucial:
				self._output.print_error(ot.errors.NO_SUCH_BOND)
			else:
				self._output.print_warn(ot.errors.NO_SUCH_BOND)

		return tuple(atom_f.get_vector_to(atom_t))

	def set_center_atom(self, a, residues=None):
		"""
        Moves all atoms such that a specific atom is in the origin of the internal coordinate system.

        Parameters
        ----------
        a : String, Integer, :class:`~toto.atom.atom` instance or list of floats. If string, the value is assumed to be a unique atom name. If integer, the value denotes the zero based index of the atom in :attr:`_atoms`. In case a list of floats is given as input, the vector defined by these three values is assumed to be the correct translation to perform.
        residues : Iterable of strings.
        	Contains the residue names that are movable. If `None`, all atoms are selected for translation.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		dt = None
		if type(a) == str:
			a = self._atoms[self.get_index(a)]
		if type(a) == int:
			a = self._atoms[a]
		if isinstance(a, atom.atom):
			dt = map(lambda x: -x, a._pos)
		if type(a) == list:
			if len(a) != 3:
				self._output.print_error(ot.errors.NO_CARTESIAN_VECTOR)
			dt = a
		if dt is None:
			self._output.print_error(ot.errors.UNKNOWN_VECTOR)

		for a in self._atoms:
			if residues is None or a.get_residue() in residues:
				a.translate(dt)
		return self

	def rotate(self, n, angle, residues=None):
		"""
		Rotates all atoms.

		Parameters
		----------
		n : Iterable of floats
      		The normal vector of the rotation. 
      	angle : Float
      		The angle to rotate by in radians.
      	residues : Iterable of strings, None
      		Rotate only atoms that are part of the given residues. If `None`, all residues are rotated.

      	Returns
      	-------
      	self
      		Allows for fluent interface.
		"""
		if residues is None:
			map(lambda x: x.rotate(n, angle), self._atoms)
		else:
			map(lambda x: x.rotate(n, angle), [_ for _ in self._atoms if _.get_residue() in residues])
		return self

	def _get_atom_lines(self):
		"""
        Extracts all lines that form an atom description from the given source lines in :attr:`_lines`.

        Returns
        -------
        List
        	Strings containing atom definitions in plain PDB format. Lines contain terminal newline characters.
		"""
		relevant = [_ for _ in self._lines if _.startswith('HETATM')]
		relevant += [_ for _ in self._lines if _.startswith('ATOM ')]
		return relevant

	def get_residues(self):
		"""
        Lists all residue names given in the input lines held in :attr:`_lines`. Parsing applies relaxed standards.

  		Returns
  		-------
  		Set
  			Unique collection of residue names.
		"""
		# separate parsing with relaxed standards
		residues = [_[17:20].strip() for _ in self._get_atom_lines()]
		return set(residues)

	def _strip_bom(self):
		"""
        Removes any BOM from the beginning of the input file data in :attr:`_lines`.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		for bom in (codecs.BOM, codecs.BOM_BE, codecs.BOM_LE, codecs.BOM_UTF8, codecs.BOM_UTF16, codecs.BOM_UTF16_BE, codecs.BOM_UTF16_LE, codecs.BOM_UTF32, codecs.BOM_UTF32_BE, codecs.BOM_UTF32_LE):
			if self._lines[0].startswith(bom):
				self._lines[0] = self._lines[0][len(bom):]

		return self

	def _add_residue(self, key):
		"""
        Internal helper updating the data structures to accomodate for another residue.

        Parameters
        ----------
        key : String
        	Residue name. May be replaced by anything that has a residue name as string representation.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		if not key in self._residues:
			self._residues[str(key)] = dict()

	def _add_atom_name(self, residue, atom_name, line):
		"""
		Internal helper updating the data structures to accomodate for another atom.

		Returns
		-------
      	String or `None`
      		The previous definition line with the same atom name and the same residue. If no such line has been stored yet, the function returns `None`.
		"""
		self._add_residue(residue)
		if atom_name in self._residues[residue]:
			return self._residues[residue][atom_name]
		else:
			self._residues[residue][atom_name] = line
			return None

	def _is_left_justified(self, str):
		"""
        Checks whether a string is left-justified.

        Parameters
        ----------
        str : String
        	Input to check.

        Returns
        -------
        Boolean
		"""
		if len(str.strip()) == 0:
			return True
		return (str[0] != ' ')

	def _is_right_justified(self, str):
		"""
        Checks whether a string is right-justified.

        Parameters
        ----------
        str : String
        	Input to check.

        Returns
        -------
        Boolean
		"""
		return self._is_left_justified(str[::-1])

	def check_all(self):
		"""
        Checks the input lines in :attr:`_lines` for syntactic errors.

        Returns
        -------
        Boolean
        	True if everything is correct.
		"""
		if self._valid is None:
			if self._check_keywords():
				self._valid = self._check_HETATM()
			else:
				self._valid = False
		return self._valid

	def _found_error(self, idx, msg, got):
		"""
        Helper function for error handling.

        Parameters
        ----------
        idx : Integer
        	One based line number.
        msg : String
        	Hint on what is wrong with this line.
        got : String
        	Hint on what was expected by the parser in this line.

        Returns
        -------
        False
		"""
		self._output.print_warn(ot.errors.PARSE_ERROR).add_level()
		msg = 'line %d: %s' % (idx, msg)
		if not got is None:
			msg += ', got >%s<' % got
		self._output.print_info(msg).del_level()
		return False

	def _check_keywords(self):
		"""
        Checks all keywords in the input data in :attr:`_lines` for validity.

        Returns
        -------
        Boolean
        	True if everything is correct.
		"""
		success = True
		for idx, line in enumerate(self._lines):
			if len(line) != 81:
				success = self._found_error(idx+1, 'expected 80 characters in line', None)
			if line[0:6] not in [
				'HEADER', 'OBSLTE', 'TITLE ', 'SPLT  ', 'CAVEAT',
				'COMPND', 'SOURCE', 'KEYWDS', 'EXPDTA', 'NUMMDL',
				'MDLTYP', 'AUTHOR', 'REVDAT', 'SPRSDE', 'JRNL  ',
				'REMARK', 'DBREF ', 'DBREF1', 'DBREF2', 'SEQADV',
				'SEQRES', 'MODRES', 'HET   ', 'FORMUL', 'HETNAM',
				'HETSYN', 'HELIX ', 'SHEET ', 'SSBOND', 'LINK  ',
				'CISPEP', 'SITE  ', 'CRYST1', 'MODEL ', 'ATOM  ',
				'ANISOU', 'TER   ', 'HETATM', 'ENDMDL', 'CONECT', 
				'MASTER','END   '] and not line[0:5] in ['ORIGX', 'SCALE', 'MTRIX']:
				success = self._found_error(idx+1, 'unknown keyword', line[0:6])
				print line.strip()[0:4]

		return success

	def _check_HETATM(self):
		"""
        Strict lint of all HETATM lines in the input data in :attr:`_lines`. 

        Prints messages on invalid lines via :attr:`_output`.

        Returns
        -------
        Boolean
      		True if everything is correct.
		"""
		success = True
		relevant = [(idx, line) for idx, line in enumerate(self._lines) if line.startswith('HETATM')]

		for idx, line in relevant:
			if not line[6] is ' ':
				success = self._found_error(idx+1, 'expected space at character 7', line[6])
			for match in re.finditer('[^0-9 ]+', line[6:11]):
				success = self._found_error(idx+1, 'only integer atomic serial numbers allowed', line[6:11])
			atom_name = line[12:16]
			residue = line[17:20]
			match = self._add_atom_name(residue, atom_name, idx+1)
			if not match is None:
				success = self._found_error(idx+1, 'non unique atom names (previous definition at line %d)' % match, None)
			try:
				coord_x = float(line[30:38])
			except ValueError:
				success = self._found_error(idx+1, 'conversion to float for x coordinate failed', line[30:38])
			try:
				coord_y = float(line[38:46])
			except ValueError:
				success = self._found_error(idx+1, 'conversion to float for y coordinate failed', line[38:46])
			try:
				coord_y = float(line[46:54])
			except ValueError:
				success = self._found_error(idx+1, 'conversion to float for z coordinate failed', line[46:54])
			try:
				coord_y = float(line[54:60])
			except ValueError:
				success = self._found_error(idx+1, 'conversion to float for occupancy failed', line[54:60])
			try:
				coord_y = float(line[60:66])
			except ValueError:
				success = self._found_error(idx+1, 'conversion to float for temperature factor failed', line[60:66])
			if not self._is_left_justified(line[72:76]):
				success = self._found_error(idx+1, 'Segment identifier not left left-justified', line[72:76])
			if not self._is_right_justified(line[76:78]):
				success = self._found_error(idx+1, 'Element symbol is not left right-justified', line[76:78])
			try:
				charge = int(line[78])
			except ValueError:
				success = self._found_error(idx+1, 'Non-integer charge', line[78])
			if line[79] not in ['+', '-']:
				success = self._found_error(idx+1, 'Invalid charge sign', line[79])

		return success

	def to_pdb_file(self, filename):
		"""
       	Write string representation of the parsed PDB information to file.

       	.. todo::
       		Error checking on file opening.

       	Parameters
       	----------
       	filename : String
       		The target filename.

       	Returns
       	-------
       	self
       		Allows for fluent interface.
		"""
		fh = open(filename, 'w')
		fh.write(self.get_pdb_string(renumber=True))
		return self

	def to_psf_file(self, filename):
		"""
        Write string representation of the parsed PSF information to file.

       	.. todo::
       		Error checking on file opening.

        Parameters
        ----------
        filename : String
        	The target filename.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		fh = open(filename, 'w')
		fh.write(self.get_psf_string())
		return self

	def translate_residue(self, residue_name, vector):
		"""
        Shifts all atoms for a specific residue after consistency checks.

        Parameters
        ----------
        residue_name : String
        	The residue that should be translated.
        vector : Iterable of floats
        	Iterable of length 3.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		if residue_name not in self.get_residues():
			self._output.print_info('Requested to move residue %s' % residue_name).add_level()
			self._output.print_error(ot.errors.NO_SUCH_RESIDUE)
		for a in self._atoms:
			if a.get_residue() == residue_name:
				a.translate(vector)
		return self

	def translate(self, vector):
		"""
        Shifts all atoms.

        Parameters
        ----------
        vector : Iterable of floats
        	Iterable of length 3.

        Returns
        -------
        self
        	Allows for fluent interface.
		"""
		for a in self._atoms:
			a.translate(vector)
		return self

def join_pdb(one, another, purge_metadata=False):
	"""
	Joins two PDB files while maintaining topology information from associated PSF files.

	Parameters
	----------
	one : :class:`PDBFile`
		Instance to be merged that will occurr first in the resulting file.
	another : :class:`PDBFile`
		Instance to be merged that will come second in the output file.
	purge_metadata : Boolean
		Whether to remove any entries from the output file that does not describe an atom position except the ones that precede any atom definition. Usually, this purges any metadata from `another`.

	Raises
	------
	ValueError :
		If any of the associated topologies contain unrecoverable errors.

	Returns
	-------
	:class:`PDBFile`
		Combined file with corrected PSF topology data.
	"""
	if not one._valid and another._valid:
		raise ValueError('Unable to join: invalid input files.')

	if not one._parsed:
		one.parser()
	if not another._parsed:
		another.parse()

	lines = [_ for _ in one._lines if not _.startswith('END')] + another._lines
	c = pdbfile(lines, one._output)
	c._residues = copy.deepcopy(one._residues)
	c._residues.update(another._residues) 
	c._valid = True
	c._atoms = [copy.deepcopy(_) for _ in one._atoms] + [copy.deepcopy(_) for _ in another._atoms]
	c._parsed = True

	# psf
	shift = len(one._atoms)
	c._psf = copy.deepcopy(one._psf)
	n_atoms = copy.deepcopy(another._psf._atoms)
	for atom in n_atoms:
		atom.set_psf_number(atom.get_psf_number()+shift)

	c._psf._atoms += n_atoms
	for a, b in another._psf._bonds:
		c._psf._bonds.append((a+shift, b+shift))
	for a, b, d in another._psf._angles:
		c._psf._angles.append((a+shift, b+shift, d+shift))
	for a, b, d, e in another._psf._dihedrals:
		c._psf._dihedrals.append((a+shift, b+shift, d+shift, e+shift))
	for a, b, d, e in another._psf._impropers:
		c._psf._impropers.append((a+shift, b+shift, d+shift, e+shift))
	
	if purge_metadata:
		c.purge_metadata(except_first=True)

	return c



#!/usr/bin/env python -u
"""
Calculates the viscosity of a fluid. 


"""
# system modules
import argparse
import itertools
import math
import numpy as np

# custom modules
import f_core as fc
import output_tracker as ot

parser = argparse.ArgumentParser()
parser.add_argument('topology', type=str, help='The topology to work with.')
parser.add_argument('coordinates', type=str, help='The coordinates to work with.')
parser.add_argument('velocities', type=str, help='The velocities to work with.')
parser.add_argument('temperature', type=float, help='The simulation temperature.')
parser.add_argument('--selection_frame', type=int, help='Input frame to select on (zero-based).', default=0)
parser.add_argument('--atom-charmm', type=str, help='CHARMM selection syntax.', default=None)

def get_lj_force(e1, e2, s1, s2, r):
	e = math.sqrt(e1*e2)
	s = (s1+s2)/2
	return -24*e*(s**6/r**7-2*s**12/r**13)

def get_coulomb_force(q1, q2, r):
	return 1389.355 * q1 * q2 / r**2

def get_parameters(atomtype):
	data = None
	if atomtype == 'FCF3':
		data = [-0.0240, 1.3400, -0.12]
	if atomtype == 'CCF3':
		data = [-0.0780, 2.0400, 0.36]
	if atomtype == 'FCF2':
		data = [-0.1050, 1.6300, -0.185]
	if atomtype == 'CCF2':
		data = [-0.0420, 2.0500, 0.37]
	if data is None:
		raise ValueError('Unknown atom type.')
	data[1] *= 2**(5./6)
	return data

def get_force(universe, group1, group2, frame, axis, output):
	# CHARMM force field: LJ and Coulomb
	#if set(group1) == set(group2):
	#	output.print_error(messages.IDENTICAL_GROUPS)

	force = np.array((0,0,0))
	g1pos = universe.atoms[group1].get_positions()
	g2pos = universe.atoms[group2].get_positions()
	for idx1, atom1 in enumerate(group1):
		e1, s1, q1 = get_parameters(universe.atoms[atom1].type)
		for idx2, atom2 in enumerate(group2):
			e2, s2, q2 = get_parameters(universe.atoms[atom2].type)
			d = g1pos[idx1]-g2pos[idx2]
			n = np.linalg.norm(d)
			f = get_lj_force(e1, e2, s1, s2, n) + get_coulomb_force(q1, q2, n)
			f *= d / n
			force += f

	return force[axis]

def get_velocity(universe, group, frame, output):
	"""
	Extracts the velocity of a molecule from the trajectory.
	
	Returns
	-------
	Iterable :
		Cartesian representation of velocities in Angstrom/ps.
	"""
	try:
		d = universe.atoms[group].get_velocities(ts=frame, copy=True)
	except:	
		# TODO: check for DCD file format
		d = universe.atoms[group].get_positions() * 20.45482706
	m = universe.atoms[group].masses()
	total = sum(map(lambda x: x[0]*x[1], zip(d, m)))
	return total / (len(group) * sum(m))

def get_distance(universe, group1, group2, frame, axis, output):
	r1 = universe.atoms[group1].centerOfMass()
	r2 = universe.atoms[group2].centerOfMass()
	d = r1-r2
	return abs(d[axis])

def get_mass(universe, group, output):
	return sum(universe.atoms[group].masses())

def sigma(coordinates, velocities, groups, frame, alpha, beta, output):
	"""
	Calculates sigma.

	Parameters
	----------
	coordinates : :class:`MDAnalysis.Universe`
		The trajectory to work on.
	velocities : :class:`MDAnalysis.Universe`
		The velocities to work with.
	groups : Tuple of Tuples
		All groups (either molecules or atoms) that are being considered as moving entity. Single atom indices have to be given as tuples, as well. Indices are zero-based.
	frame : Integer
		The timestep to work on. Zero-based.
	alpha : Integer
		The first axis to select. (0 = x, 1 = y, 2 = z)
	beta : Integer
		The second axis to select. (0 = x, 1 = y, 2 = z)
	output : :class:`output_tracker.output_tracker`
		Instance for logging purposes. 

	Returns
	-------
	sigma :
		In units of amu m^2/s^2.
	"""
	value = 0
	for idx, group in enumerate(groups):
		velocity = get_velocity(velocities, group, frame, output)
		value += velocity[alpha] * velocity[beta] * get_mass(coordinates, group, output)
		for group_idx in range(idx + 1, len(groups)):
			force = get_force(coordinates, group, groups[group_idx], frame, alpha, output)
			distance = get_distance(coordinates, group, groups[group_idx], frame, beta, output)
			value += force * distance
	exit(2)

	value /= len(groups)
	return value

def P(coordinates, velocities, groups, frame, alpha, beta, output):
	value = sigma(coordinates, velocities, groups, frame, alpha, beta, output) + sigma(coordinates, velocities, groups, frame, beta, alpha, output)
	value /= 2
	if alpha == beta:
		for gamma in (0, 1, 2):
			value -= sigma(coordinates, velocities, groups, frame, gamma, gamma, output)

	return value

class group(object):
	def __init__(self, coordinates, indices):
		self._masses = coordinates.atoms[indices].masses()
		self._mass = sum(self._masses)
		self._params = [get_parameters(coordinates.atoms[_].type) for _ in indices]
		self._es = np.array([_[0] for _ in self._params])
		self._ss = np.array([_[1] for _ in self._params])
		self._qs = np.array([_[2] for _ in self._params])
		self._indices = indices

def P_complete(coordinates, velocities, groups, framenum, output):
	# extract velocities and coordinates on a group basis
	cor = coordinates.trajectory[framenum][:]
	vel = velocities.trajectory[framenum][:]

	n = len(groups)
	g_mass = np.zeros(n)
	g_coord = np.zeros([n, 3])
	g_vel = np.zeros([n, 3])
	for idx, group in enumerate(groups):
		g_mass[idx] = group._mass
		g_coord[idx] = np.sum(cor[group._indices].transpose() * group._masses, axis=1)/group._mass
		g_vel[idx] = np.sum(vel[group._indices].transpose() * group._masses, axis=1)/group._mass

	# calculate forces and distances
	g_fd = np.zeros([n, n, 3])
	for i in range(n):
		g1_cor = cor[groups[i]._indices]
		g_fd[i+1:, i] = g_coord[i]-g_coord[i+1:]
		for j in range(i+1, n):
			if np.linalg.norm(g_fd[j, i]) > 15:
				continue
			g2_cor = cor[groups[j]._indices]
			for idx, atom in enumerate(g1_cor):
				d = g2_cor - atom
				r = np.linalg.norm(d, axis=1)

				es = np.sqrt(groups[i]._es[idx] * groups[j]._es)
				ss = (groups[i]._ss[idx] + groups[j]._ss)/2
				qs = groups[i]._qs[idx] * groups[j]._qs
				res = -24*es*(ss**6/r**7-2*ss**12/r**13)*4.1868
				res += 1389.355 * qs / r**2
				d = d.transpose()/r
				g_fd[i, j] += np.linalg.norm(res * d, axis=1)

	# calculate sigmas
	sigma = np.zeros([3, 3])
	for alpha in range(3):
		for beta in range(3):
			sigma[(alpha, beta)] += np.sum(g_vel[:,alpha] * g_vel[:,beta] * g_mass)
			for i in range(n):
				for j in range(i+1, n):
					sigma[(alpha, beta)] += g_fd[i, j, alpha] * g_fd[j, i, beta]
	sigma /= n

	# calculate P
	P = np.zeros([3, 3])
	for alpha in range(3):
		for beta in range(3):
			P[alpha, beta] = (sigma[alpha, beta]+sigma[beta, alpha])/2
			if alpha==beta:
				P[alpha, beta] -= sum([sigma[_, _] for _ in range(3)])

	return P

def main():	
	"""
	Wrapper function.
	"""
	output = ot.output_tracker()
	mda = fc._require_mda(output)
	args = parser.parse_args()

	# messages
	messages = fc.messages

	# read input
	output.print_info('Reading source files.').add_level()
	output.print_info('Opening file %s.' % args.topology)
	output.print_info('Opening file %s.' % args.coordinates)
	output.print_info('Opening file %s.' % args.velocities)

	# create Universe
	fc.mda_set_periodic(True, output)
	u_c = fc.mda_load_universe(args.topology, args.coordinates, output)
	output.print_info('Coordinate file holds %d frames.' % len(u_c.trajectory))
	u_v = fc.mda_load_universe(args.topology, args.velocities, output)
	output.print_info('Velocity file holds %d frames.' % len(u_v.trajectory))
	if len(u_v.trajectory) != len(u_c.trajectory):
		output.print_error(messages.VEL_COORD_MISMATCH)
	else:
		output.del_level()

	# build groups
	resids = set([_.resid for _ in u_c.atoms])
	output.print_info('Found %d residues. Using them as groups.' % len(resids))
	groups = []
	for resid in resids:
		groups.append(group(u_c, [_.number for _ in u_c.atoms if _.resid == resid]))

	# running calculation of eta
	history = np.zeros((len(u_c.trajectory), 3, 3))
	history[0,0:,0:] = P_complete(u_c, u_v, groups, 0, output)
	b = [history[0,0,1]]
	eta = 0
	for frame in xrange(1, len(u_c.trajectory)):
		#eta += np.sum(P_complete(u_c, u_v, groups, frame, output)*cache)
		#print frame, eta*u_c.trajectory[frame].volume*1.66053892*10e-4/(10*1.3806488*args.temperature)	
		history[frame,0:,0:] = P_complete(u_c, u_v, groups, frame, output)
		b.append(history[frame,0,1])
		#autocorr = np.correlate(history[:frame,0,1], history[:frame,0,1], mode='full')
		autocorr = np.correlate(b,b, mode='full')
		autocorr = autocorr[autocorr.size/2:]
		autocorr /= autocorr[0]
		print frame, sum(autocorr)*u_c.trajectory[frame].volume*1.66053892*10e-4/(10*1.3806488*args.temperature)
		#for idx, a in axes:
			#alpha, beta = a

			#eta += P(u_c, u_v, groups, frame, alpha, beta, output) * cache[idx]
			#print frame, eta*u.trajectory[frame].volume*1.66053892*10e-4/(10*1.3806488*temperature)

if __name__ == '__main__':
	main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# system modules
import sys
import unittest

# custom modules
import f_core as fc
import output_tracker as ot

class RestTestResult(unittest.TestResult):
	def __init__(self):
		unittest.TestResult.__init__(self)
		self.output = ''

	def getDescription(self, test):
		return ''

	def startTest(self, test):
		unittest.TestResult.startTest(self, test)

	def addSuccess(self, test):
		unittest.TestResult.addSuccess(self, test) 
		self.output += ':test-ok:`%s`\n' % test.shortDescription()

	def addFailure(self, test, err):
		unittest.TestResult.addFailure(self, test, err) 
		self.output += ':test-failed:`%s`\n' % test.shortDescription()

	def addError(self, test, err):
		unittest.TestResult.addError(self, test, err)
		self.output += ':test-failed:`%s`\n' % test.shortDescription()

class RestFormatter(unittest.TextTestRunner):
	
	def _makeResult(self):
		return RestTestResult()

	def run(self, suite, doc):
		s = doc.split('\n')[0].strip()
		s += '\n' + '-'*len(s) + '\n'

		result = self._makeResult()
		suite(result)
		s += result.output
		failed, errored = map(len, (result.failures, result.errors))

		s += '\n'
		return (failed+errored, result.testsRun, s)

class TestRanges(unittest.TestCase):
	""" Range selection tests """
	def setUp(self):
		self.output = ot.null_output()

	def test_single(self):
		""" Single number is parsed. """
		self.assertEqual(fc._parse_ids('2', self.output), [2])

	def test_comma(self):
		""" Comma separated numbers are parsed. """
		self.assertEqual(fc._parse_ids('2,3', self.output), [2, 3])

	def test_space(self):
		""" Space separated numbers are parsed. """
		self.assertEqual(fc._parse_ids('2 3', self.output), [2, 3])

	def test_commaspace(self):
		""" Accepts a comma followed by a space as delimiter. """
		self.assertEqual(fc._parse_ids('2, 3', self.output), [2, 3])
		
	def test_commaspacemix(self):
		""" Accepts a mix of commas and spaces as delimiter. """
		self.assertEqual(fc._parse_ids('2,3 4', self.output), [2, 3, 4])
	
	def test_range(self):
		""" Range is parsed. """
		self.assertEqual(fc._parse_ids('2-4', self.output), [2, 3, 4])

	def test_exclusion(self):
		""" Exclusion is parsed. """
		self.assertEqual(fc._parse_ids('2-4,^3', self.output), [2, 4])

	def test_single_exclusion(self):
		""" Single exclusion is valid. """
		self.assertEqual(fc._parse_ids('^2-4', self.output), [])

	def test_precedence(self):
		""" Precedence is observed. """
		self.assertEqual(fc._parse_ids('2-4,^3, 3', self.output), [2, 3, 4])

	def test_sorted(self):
		""" Result is sorted. """
		self.assertEqual(fc._parse_ids('22-24,2-4', self.output), [2, 3, 4, 22, 23, 24])

	def test_unique(self):
		""" Results are unique. """
		self.assertEqual(fc._parse_ids('2-4, 2, 3, 4', self.output), [2, 3, 4])

	def test_python_range(self):
		""" Python ranges are parsed. """
		self.assertEqual(fc._parse_ids('2:4', self.output), [2, 3])

	def test_mixed_range(self):
		""" Mixed ranges are parsed. """
		self.assertEqual(fc._parse_ids('2:4,5-6', self.output), [2, 3, 5, 6])

	def test_range_stride(self):
		""" Python stride syntax is accepted. """
		self.assertEqual(fc._parse_ids('2:5:2', self.output), [2, 4])

	def test_nonnumeric(self):
		""" Raises exception on non-numeric characters for dash syntax. """
		self.assertRaises(Exception, fc._parse_ids, ('2-a', self.output))

	def test_nonnumeric(self):
		""" Raises exception on non-numeric characters for python syntax. """
		self.assertRaises(Exception, fc._parse_ids, ('2:a', self.output))

	def test_nonnumeric(self):
		""" Raises exception on open ranges. """
		self.assertRaises(Exception, fc._parse_ids, ('2-', self.output))

	def test_nonnumeric(self):
		""" Raises exception on open python ranges. """
		self.assertRaises(Exception, fc._parse_ids, ('2::4', self.output))

	def test_unique(self):
		""" Raises exception on invalid ranges. """
		self.assertRaises(Exception, fc._parse_ids, ('2:3', self.output))

if __name__ == '__main__':
	if len(sys.argv) != 2:
		r = RestFormatter()
		modified = True
	else:
		r = unittest.TextTestRunner()
		modified = False

	failed = 0
	total = 0
	output = ''
	for testcase in (TestRanges, ):
		suite = unittest.TestSuite()
		tests = [_ for _ in dir(testcase) if _.startswith('test_')]
		if len(tests) == 0:
			continue
		for test in tests:
			suite.addTest(testcase(test))
		if modified:
			f, t, s = r.run(suite, testcase.__doc__)
			failed += f
			total += t
			output += s
		else:
			r.run(suite)

	if modified:
		print """
Test results
============

Failed tests: %d of %s
	""" % (failed, total)
		print output

		print """
Invocation
----------

Run the test script on command line to get detailed error descriptions. Without the `debug` parameter, you will get reST output.

.. code:: sh

	f_tests.py debug

Documentation
-------------

.. automodule:: f_tests
   :members:
   :private-members:
   :special-members:
   :undoc-members:

	"""


from setuptools import setup, find_packages

import conf

setup(
    name = "finley",
    version = conf.version,
    install_requires = ['MDAnalysis'],
    requires = ['MDAnalysis'],
    py_modules = ['f_core', 'atom', 'output_tracker', 'moremath', 'colorexcept'],
    long_description = '',
    author = "Guido Falk von Rudorff",
    author_email = "guido@vonrudorff.de",
    description = "Command line analyzer for large molecular dynamics trajectories with DCD support.",
    license = "GPL 2",
    keywords = "DCD, PDB, PSF",
    url = "http://guido.vonrudorff.de/finley/",
    classifiers = ['Development Status :: 4 - Beta'],

    scripts=['f_extract.py', 'f_timeseries.py'],
)